/**
 * LICENCE[[
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1/CeCILL 2.O
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is kelis.fr code.
 *
 * The Initial Developer of the Original Code is 
 * samuel.monsarrat@kelis.fr
 *
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * sylvain.splinelli@kelis.fr
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * or the CeCILL Licence Version 2.0 (http://www.cecill.info/licences.en.html),
 * in which case the provisions of the GPL, the LGPL or the CeCILL are applicable
 * instead of those above. If you wish to allow use of your version of this file
 * only under the terms of either the GPL, the LGPL or the CeCILL, and not to allow
 * others to use your version of this file under the terms of the MPL, indicate
 * your decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL, the LGPL or the CeCILL. If you do not
 * delete the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL, the LGPL or the CeCILL.
 * ]]LICENCE
 */

/** SCENARI Slide-show presentation manager 
 */
var scPresMgr = {
	/* Paramétrage du Mgr */
	fTocPath : null,
	fTocLnksPath : null,
	fSldFraPath : null,
	fNavPath : null,
	fToolsPath : null,

	/* Paths standards. */
	fContainerFilter : ".ssContainer",
	fContainerBlocksRootPath : "des:.ssBkRoot",
	fBlkCoPath : "des:.ssBkCo",
	fCutableFilter : ".ssCutable|p|ol|ul|li",
	fUncutableFilter : ".ssUncutable|tr",

	/* Constantes de stylage. */
	fSsCssName : "modeSS",
	fSsClassPrefix : "tpl",

	/* Constantes pour les animations. */
	/** Largeur mini du slide */
	fBlockMinWidth : 500,
	/** Ratio de marge max par rapport à la largeur du container */
	fBlockMaxMargin : 0.3,
	/** Ratio d'espace libre à placer au dessus du contenu */
	fBlockTopSpace : 0.3,
	/** Opacité pour les masks des steps. */
	fStepMaskOpacity : 0.9,
	
	/* Mémoires */
	fToc : null,
	fFra : null,
	fNav : null,
	fTocLnks : null,
	fSldUrls : null,
	fSldHome : null,
	fContainerFilterComp : null,
	fContainerBlocksRootPathComp: null,
	fBlkCoPathComp:null,
	fCutableFilterComp : null,
	fListeners : [],
	fToolElts : [],
	fSldPosStyledElts : [],
	fEnableEffects : true,
	
	/* Etats */
	fCurrSld : null,

	/* Localisations */
	fStrings : {en:["previous","previous (left arrow)",       //0
	            "next","next (right arrow)",                  //2
	            "close","close the zoom (ESC)",               //4
	            "WARNING: The Firebug extention is active.\nIt is advisable to deactivate Firebug while viewing this slide-show.","WARNING: This slide-show is not compatible with Internet Explorer 6.\nPlease use a more recent browser.",//6
	            "HTML mode","Switch to plain HTML mode (H)", //8
	            "Are you shure you want to switch to a plain HTML version of this slide-show?","Effects", //10
	            "Activate visual effects (F)","Deactivate visual effects (F)"], //12
	            fr:["Précédant","Précédant (flèche gauche)",
	            "Suivant","Suivant (flèche droite)",
	            "Fermer","Fermer le zoom (ESC)",
	            "ATTENTION : L'extension Firebug est actif.\nIl est conseillé de désactiver Firebug pour visualiser ce diaporama.","ATTENTION : Ce diaporama n'est pas compatible avec Internet Explorer 6.\nVeuillez utiliser un navigateur plus récent.",
	            "Mode HTML","Basculer en mode HTML simple (H)",
	            "Etes-vous sûr de vouloir basculer vers une version HTML simple de ce diaporama ?","Effets",
	            "Activer les effets visuels (F)","Désactiver les effets visuels (F)"],
	            es:["anterior","anterior (flecha izquierda)",
	            "siguiente","siguiente (flecha derecha)",
	            "cerrar","cerrar el zoom (ESC)",
	            "ATENCIÓN: La extensión Firebug esta activa.\nSe recomienda desactivar Firebug mientras consulte esa presentación","ATENCIÓN : Esta presentación con diapositivas no es compatible con Internet Explorer 6.\nPor favor utilice un navegador más reciente.",
	             "Modo HTML","Pasar en modo HTML simple (H)",
	            "¿ Está seguro de querer pasar a la versión simple HTML de este diaporama ?","Efectos",
	            "Activar los efectos visuales (F)","Desactivar los efectos visuales (F)"]
	            },
	fNavie6 : parseFloat(scCoLib.userAgent.substring(scCoLib.userAgent.indexOf("msie")+5)) < 7,

	/* === Public ============================================================= */
	/** Init the manager
	 * @param pSldFraPath : path to the slide frame.
	 * @param pNavPath : path to the navigation bar.
	 * @param pTocLnksPath : path to all toc entries.
	 */
	init : function(pSldFraPath, pNavPath, pTocLnksPath) {
		//scCoLib.util.log("scPresMgr.init");
		if (window.location.search.indexOf("mode=html")>-1) return;
		if (this.fNavie6){ // Slideshow mode not compatible with IE6.
			if(document.cookie.indexOf("ie6Warn=true")<0) alert(this.xGetStr(7));
			document.cookie = "ie6Warn=true";
			return;
		}
		this.fSldFraPath = pSldFraPath;
		this.fNavPath = pNavPath;
		this.fTocLnksPath = pTocLnksPath;
		//Compilations paths et filters
		this.fContainerFilterComp = scPaLib.compileFilter(this.fContainerFilter);
		this.fContainerBlocksRootPathComp = scPaLib.compilePath(this.fContainerBlocksRootPath);
		this.fBlkCoPathComp = scPaLib.compilePath(this.fBlkCoPath);
		this.fCutableFilterComp = scPaLib.compileFilter(this.fCutableFilter);
		//Init Listeners
		this.fListeners = {};
		this.fListeners['onSldLoad'] = new Array();
		this.fListeners['onSldShow'] = new Array();
		this.fListeners['onKeyPress'] = new Array();
		//Init liste elements outils
		this.fToolElts = new Array();
		//Init liste elements stylés avec la position actuel dans le slide-show (FirstSlide LastSlide, FirstStep, LastStep)
		this.fSldPosStyledElts = new Array();

		scOnLoads[scOnLoads.length] = this;
	},
	setContainerFilter : function(pContainerFilter) {
		this.fContainerFilter = pContainerFilter;
	},
	setCutableFilter : function(pCutableFilter) {
		this.fCutableFilter = pCutableFilter;
	},
	setUncutableFilter : function(pUncutableFilter) {
		this.fUncutableFilter = pUncutableFilter;
	},
	setContainerBlocksRootPath : function(pContainerBlocksRootPath) {
		this.fContainerBlocksRootPath = pContainerBlocksRootPath;
	},
	setBlkCoPath : function(pBlkCoPath) {
		this.fBlkCoPath = pBlkCoPath;
	},
	/** Set path of optional tools element (if defined extra buttons will be added such as the ability to switch to HTML mode). */
	setToolsPath : function(pToolsPath) {
		this.fToolsPath = pToolsPath;
	},
	/** Set paths of elements that constitute the zoom frame. */
	setZoomPaths : function(pZoomFramePath,pZoomContentPath) {
		this.fZoomFramePath = pZoomFramePath;
		this.fZoomContentPath = pZoomContentPath;
	},
	/** register a path as a slide-position styled element. */
	addSlidePositionStyledPath : function(pPath) {
		this.fSldPosStyledElts.push(pPath);
	},
	/** register a listener. */
	register : function(pType, pFunc) {
		this.fListeners[pType].push(pFunc);
	},
	/** Enable / Disable auto tool hiding */
	enableToolHider : function(pEnable) {
		this.fToolHider = pEnable;
		if (!pEnable) this.xShowTools();
	},
	/** Enable / Disable visual effects */
	enableEffects : function(pEnable) {
		this.fEnableEffects = pEnable;
		if (this.fBtnEfcts) this.fBtnEfcts.style.display = (this.fEnableEffects ? "none" : "");
		if (this.fBtnNoEfcts) this.fBtnNoEfcts.style.display = (this.fEnableEffects ? "" : "none");
	},
	/** Enable / Disable visual effects */
	toggleEffects : function() {
		this.enableEffects(!this.fEnableEffects);
	},
	/** register a node as a tool (auto hidden elements). */
	addToolElt : function(pNode) {
		this.fToolElts.push(pNode);
	},
	/** Main OnLoad function called by the SCENARI framework. */
	onLoad : function() {
		try{
			//scCoLib.util.log("scPresMgr.onLoad");
			//Find Slideshow frame
			this.fSldFra = scPaLib.findNode(this.fSldFraPath);
			//Enable fSsCssName CSS
			var vCss = null;
			for(var i = 0; (vCss = document.getElementsByTagName("link")[i]); i++) {
				if(vCss.getAttribute("rel").indexOf("style") != -1 && vCss.getAttribute("title")) {
					vCss.disabled = true; //Bug IE : obliger de désactiver en premier PUIS réactiver.
					if(vCss.getAttribute("title").indexOf(this.fSsCssName) != -1) vCss.disabled = false;
				}
			}
			//Init toc
			this.fTocLnks = scPaLib.findNodes(this.fTocLnksPath);
			this.fBtnHome = this.fTocLnks.shift();
			this.fBtnHome.onclick = function(){scPresMgr.loadSld(-1);return false;};
			this.fSldUrls = [];
			for (var i=0; i < this.fTocLnks.length; i++){
				var vLnk = this.fTocLnks[i];
				this.fSldUrls[i] = vLnk.protocol + "//" + (vLnk.hostname ? vLnk.hostname : "") + (vLnk.port ? ":" + vLnk.port : "") + (vLnk.pathname.indexOf("/")!=0 ? "/" : "") + vLnk.pathname;
				vLnk.fSldIdx = i;
				vLnk.onclick = this.sOnClickTocLnk;
			}
			//Init events
			document.onkeyup = scPresMgr.sOnKeyUp;
			this.fToolsShowTime = new Date().getTime();
			this.fHeartbeat = window.setInterval(this.sHeartbeat,1000);
			if(scCoLib.isIE) document.body.attachEvent("onmousemove", this.sMouseMgr);
			else window.addEventListener("mousemove", this.sMouseMgr,true);
			if(scCoLib.isIE) document.body.attachEvent("onmousedown", this.sMouseMgr);
			else window.addEventListener("mousedown", this.sMouseMgr,true);
			//Init nav bar
			this.fNav = scPaLib.findNode(this.fNavPath);
			this.fNav.fDefaultClass = this.fNav.className;
			this.fNav.innerHTML = ""; // Purge the nav bar
			this.fBtnPrv = this.xAddBtn(this.fNav, "btnPrv", this.xGetStr(0), this.xGetStr(1));
			this.fBtnPrv.style.visibility = "hidden";
			this.fBtnNxt = this.xAddBtn(this.fNav, "btnNxt", this.xGetStr(2), this.xGetStr(3));
			this.addToolElt(this.fNav);
			//Init Slide d'accueil
			this.fSldHome = scPaLib.findNode("chi:div.slide", this.fSldFra);
			this.fSldHome.fSldMgr = new scPresMgr.SldHomeMgr(this.fSldHome);
			this.fSldHome.fSldMgr.initSld();
			//Init optional tools 
			if (this.fToolsPath){
				this.fTools = scPaLib.findNode(this.fToolsPath);
				this.fTools.innerHTML = ""; // Purge the tools bar
				this.fBtnModeHtml = this.xAddBtn(this.fTools, "btnModeHtml", this.xGetStr(8), this.xGetStr(9));
				this.fBtnEfcts = this.xAddBtn(this.fTools, "btnEfcts", this.xGetStr(11), this.xGetStr(12));
				this.fBtnEfcts.style.display = (this.fEnableEffects ? "none" : "");
				this.fBtnNoEfcts = this.xAddBtn(this.fTools, "btnNoEfcts", this.xGetStr(11), this.xGetStr(13));
				this.fBtnNoEfcts.style.display = (this.fEnableEffects ? "" : "none");
				this.addToolElt(this.fTools);
			}
			//Init zoom container
			this.fZoom = (this.fZoomFramePath ? scPaLib.findNode(this.fZoomFramePath) : this.xAddElt("div", document.body, null));
			this.fZoom.className = this.fZoom.className + " "+this.fSsClassPrefix+"Zm"
			this.fZoom.style.visibility = "hidden";
			this.fZoom.fCo = (this.fZoomContentPath ? scPaLib.findNode(this.fZoomContentPath) : this.fZoom);
			this.fZoom.fSld = scPresMgr.xAddElt("div", this.fZoom.fCo, this.fSsClassPrefix+"ZmSld");
			this.fZoom.fFrg = this.xAddElt("div", this.fZoom.fCo, this.fSsClassPrefix+"ZmFrg");
			this.fZoom.fBtnCls = this.xAddBtn(this.fZoom.fCo, "btnZmCls", this.xGetStr(4), this.xGetStr(5));
			//Init slide-position styled elements
			for (var i in this.fSldPosStyledElts){
				this.fSldPosStyledElts[i] = scPaLib.findNode(this.fSldPosStyledElts[i]);
				if (this.fSldPosStyledElts[i]) this.fSldPosStyledElts[i].fBaseClass = this.fSldPosStyledElts[i].className;
			}
			//Enable tool auto hiding
			this.enableToolHider(true);
			//Affichage du slide d'accueil (rendu instantanné)
			this.fSwitchSldTask.initTask(this.fSldHome, "first", true);
			if (!scCoLib.fDebug && window.console && window.console.firebug) alert(this.xGetStr(6));
		}catch(e){scCoLib.util.logError("ERROR scPresMgr.onLoad : ", e);}
	},
	loadSortKey : "A",
	/** Load a slide by index id. 
	 * @param pFromStart affiche le slide du début (true) ou de la fin (false).
	 * @param pInstantResult si true précipite le résultat instannément, sans animation.
	 * @return false si echec (pas de slide d'idx pIdx, ...)
	 */
	loadSld : function(pIdx, pFromStart, pInstantResult) {
		scCoLib.util.log("scPresMgr.loadSld: "+pIdx);
		this.xResetFocus();
		return this.xGotoSlide(pIdx, pFromStart, pInstantResult);
	},
	/** Returns the current slide. */
	getCurrSld : function(){
		return scPresMgr.fSwitchSldTask.fNewSld || this.fCurrSld;
	},
	/** Redessinement de la zone des slides (suite à resize notamment). */
	redrawSlideZone : function(){
		//Si besoin fixer la zone porteuse des slides this.fSldFra en JS... (IE6)
		if(this.fCurrSld) this.fCurrSld.fSldMgr.redrawSld();
		var vFutureSld =  scPresMgr.fSwitchSldTask.fNewSld;
		if(vFutureSld && ! vFutureSld.fSldMgr.fIsLoaded) vFutureSld = null;
		var vSld = this.fSldFra.firstChild;
		while(vSld) {
			if(vSld === this.fCurrSld || vSld === vFutureSld ) {
				vSld.fSldMgr.redrawSld();
			} else {
				vSld.fSldMgr.invalidateSize();
			}
			vSld = vSld.nextSibling;
		}
	},
	/** Show the passed ressource in the zoom. */
	showZoom : function(pRes) {
		return this.xShowZoom(pRes);
	},
	/** Returns true if the presentation has a next step / slide. */
	hasNxt : function() {
		var vSld = this.getCurrSld();
		if (vSld && vSld.fSldMgr) {
			return (vSld.fSldMgr.hasNxt() ? true : !vSld.fSldMgr.isLastSld());
		} else return false;
	},
	/** Returns true if the presentation has a previous step / slide. */
	hasPrv : function() {
		var vSld = this.getCurrSld();
		if (vSld && vSld.fSldMgr) {
			return (vSld.fSldMgr.hasPrv() ? true : !vSld.fSldMgr.isFirstSld());
		} else return false;
	},

	/* === Private ============================================================ */
	
	/** Move forward 1 step / slide.
	 * @param pSkip if true move to beginning of next slide.
	 * @return false si l'action a échouée (chargement en cours du slide).
	 */
	xNxt : function(pSkip) {
	
		if(this.fZoom.fAct && !pSkip){
			if(this.fZoom.fSldMgr && !this.fZoom.fSldMgr.fIsLoaded) return false;
			if(!this.fZoom.fSldMgr || this.fZoom.fSldMgr && !this.fZoom.fSldMgr.goToNxt()) this.xHideZoom();
		} else {
			this.xHideZoom();
			var vFromSld =  scPresMgr.fSwitchSldTask.fNewSld || this.fCurrSld;
			if( ! vFromSld || ! vFromSld.fSldMgr.fIsLoaded) return false;
			if (pSkip || ! vFromSld.fSldMgr.goToNxt()) {
				return this.xGotoSlide(vFromSld.fSldMgr.fSldIdx+1, true, false);
			}
		}
		return true;
	},
	/** Move back 1 step / slide.
	 * @param pSkip if true move to beginning of previous slide.
	 * @return false si l'action a échouée (chargement en cours du slide).
	 */
	xPrv : function(pSkip) {
		if(this.fZoom.fAct && !pSkip){
			if(this.fZoom.fSldMgr && !this.fZoom.fSldMgr.fIsLoaded) return false;
			if(!this.fZoom.fSldMgr || this.fZoom.fSldMgr && !this.fZoom.fSldMgr.goToPrv()) this.xHideZoom();
		} else {
			var vFromSld =  scPresMgr.fSwitchSldTask.fNewSld || this.fCurrSld;
			if( ! vFromSld ||  ! vFromSld.fSldMgr.fIsLoaded) return false;
			if (pSkip || ! vFromSld.fSldMgr.goToPrv()) {
				return this.xGotoSlide(vFromSld.fSldMgr.fSldIdx-1, (pSkip ? true : false), false);
			}
		}
		return true;
	},
	/** xUpdateGui */
	xUpdateGui : function() {
		scCoLib.util.log("scPresMgr.xUpdateGui");
		var vSld = this.getCurrSld();
		if (vSld && vSld.fSldMgr) {
			this.fBtnNxt.style.visibility = (vSld.fSldMgr.hasNxt() ? "" : (!vSld.fSldMgr.isLastSld() ? "" : "hidden"));
			this.fBtnPrv.style.visibility = (vSld.fSldMgr.hasPrv() ? "" : (!vSld.fSldMgr.isFirstSld() ? "" : "hidden"));
		}
//		window.setTimeout(function(){scPresMgr.xUpdateSldPosStyledElts()},1000);
		this.xUpdateSldPosStyledElts();
	},
	/** xUpdateSldPosStyledElts */
	xUpdateSldPosStyledElts : function() {
		// Add slide-postion classes on registed elements if needed.
		var vSld = this.getCurrSld();
		if (vSld && vSld.fSldMgr) {
			for (var i in this.fSldPosStyledElts){
				var vNewClass = this.fSldPosStyledElts[i].fBaseClass + (vSld.fSldMgr.isFirstSld() ? " "+this.fSsClassPrefix+"FirstSlide" + (!vSld.fSldMgr.hasPrv() ? " "+this.fSsClassPrefix+"FirstStep" : "") : (vSld.fSldMgr.isLastSld() ? " "+this.fSsClassPrefix+"LastSlide" + (!vSld.fSldMgr.hasNxt() ? " "+this.fSsClassPrefix+"LastStep" : "") : ""));
				if (this.fSldPosStyledElts[i].className != vNewClass) this.fSldPosStyledElts[i].className = vNewClass;
			}
		}
	},
	/** Navigation vers un slide. 
	 * @param pFromStart affiche le slide du début (true) ou de la fin (false).
	 * @return false si echec (pas de slide d'idx pIdx, ...)
	 */
	xGotoSlide : function(pIdx, pFromStart, pInstantResult) {
		scCoLib.util.log("scPresMgr.xGotoSlide: "+pIdx);
		try{
			//Borne pIdx selon les limites du slide show
			if(pIdx < -1) return false;
			if(pIdx >= this.fSldUrls.length) return false;
		
			var vFutureSld =  scPresMgr.fSwitchSldTask.fNewSld;
			var vFutureIdx = vFutureSld ? vFutureSld.fSldMgr.fSldIdx : -99;
			var vCurrIdx = this.fCurrSld ? this.fCurrSld.fSldMgr.fSldIdx : -99;
			if(vCurrIdx == pIdx) {
				//On redemande le slide en cours.
				//On kill un éventuel début de transition vers un autre slide.
				if(vFutureSld) scPresMgr.fSwitchSldTask.rollbackTask();
				var vCurrSldMgr = this.fCurrSld.fSldMgr;
				var vBlock = pFromStart ? vCurrSldMgr.getFirstBlock() : vCurrSldMgr.getLastBlock();
				scPresMgr.fSwitchBlkTask.initTask(vCurrSldMgr.getCurrBlk(), vBlock, pFromStart? "F" : "L", pInstantResult);
			} else if(vFutureIdx == pIdx) {
				//On redemande le slide qui va venir
				scPresMgr.fSwitchSldTask.precipitateEndTask();
				var vCurrSldMgr = this.fCurrSld.fSldMgr;
				var vBlock = pFromStart ? vCurrSldMgr.getFirstBlock() : vCurrSldMgr.getLastBlock();
				scPresMgr.fSwitchBlkTask.initTask(vCurrSldMgr.getCurrBlk(), vBlock, pFromStart? "F" : "L", pInstantResult);
			} else {
				//Recherche le slide dans le cache
				var vSld = scPresMgr.fSldFra.firstChild;
				while(vSld) {
					if(vSld.fSldMgr.fSldIdx == pIdx) {
						//Trouvé
						scPresMgr.fSwitchSldTask.initTask(vSld, pFromStart? "first" : "last", pInstantResult);
						return true;
					}
					vSld = vSld.nextSibling;
				}
				//Pas trouvé dans le cache, on charge.
				scPresMgr.fSwitchSldTask.initTask(this.xBuildSlide(pIdx), pFromStart? "first" : "last", pInstantResult);
			}
			return true;
		}catch(e){scCoLib.util.logError("ERROR scPresMgr.xGotoSlide: ",e);}
	},
	/** Affecte le nouveau slide "stabilisé" et devient la nouvelle référence.
	 * Gère le cache des slides en fonction de cette nouvelle position.
	 */
	xSetCurrSlide : function(pNewCurrSlide) {
		try{
			scPresMgr.fCurrSld = pNewCurrSlide;
			var vNewIdx = pNewCurrSlide.fSldMgr.fSldIdx;
			var vNextIdx = vNewIdx + 1;
			var vPrevIdx = vNewIdx - 1;
			var vNextLimit = vNewIdx + 8;
			var vPrevLimit = vNewIdx - 2;
			var vIsNextOk = false;
			var vIsPrevOk = false;
			var vSld = this.fSldFra.firstChild;
			while(vSld) {
				var vNext = vSld.nextSibling;
				var vSldIdx = vSld.fSldMgr.fSldIdx;
				if(vSldIdx == vPrevIdx) {
					vIsPrevOk = true;
				} else if(vSldIdx == vNextIdx) {
					vIsNextOk = true;
				}
				if((vSldIdx < vPrevLimit || vSldIdx > vNextLimit) && vSldIdx>=0) {
					//pas un slide de proximité, ni l'accueil, on purge.
					this.fSldFra.removeChild(vSld);
				}
				vSld = vNext;
			}
			if( ! vIsNextOk && vNextIdx >= 0 && vNextIdx < this.fSldUrls.length) {
				this.xBuildSlide(vNextIdx);
			}
			if( ! vIsPrevOk && vPrevIdx >= 0 && vPrevIdx < this.fSldUrls.length) {
				this.xBuildSlide(vPrevIdx);
			}
			this.xResetFocus();
			this.xNotifyListeners("onSldShow", this.fCurrSld);
		}catch(e){scCoLib.util.logError("ERROR scPresMgr.xSetCurrSlide: ",e);}
	},
	/** Chargement d'un slide. */
	xBuildSlide : function(pIdx) {
		//scCoLib.util.log("scPresMgr.xBuildSlide: "+pIdx);
		var vSld = this.xAddEltHidden("div", scPresMgr.fSldFra, "slide");
		vSld.fSldMgr = new scPresMgr.SldMgr(vSld, pIdx);
		vSld.fSldMgr.initSld();
		return vSld;
	},
	/** Buttons manager. */
	xBtnMgr : function(pBtn){
		switch(pBtn.fName){
			case "btnNxt":
				this.xNxt();break;
			case "btnPrv":
				this.xPrv();break;
			case "btnZmCls":
				this.xHideZoom();break;
			case "btnModeHtml":
				this.xSwitchToHtmlMode();break;
			case "btnEfcts":
				this.enableEffects(true);break;
			case "btnNoEfcts":
				this.enableEffects(false);break;
		}
		this.xResetFocus();
	},
	/** Event clavier. */
	sOnKeyUp : function(pEvent){
		var vEvent = pEvent || window.event;
		var vCharCode = vEvent.which || vEvent.keyCode;
		return scPresMgr.xKeyMgr(vCharCode);
	},
	/** scPresMgr.xKeyMgr */
	xKeyMgr : function(pCharCode){
		//scCoLib.util.log("scPresMgr.xKeyMgr: "+pCharCode);
		this.xNotifyListeners("onKeyPress", pCharCode);
		switch(pCharCode){
			case 39://right
			case 40://down
			case 32://space
			case 78://n
				this.xNxt(); return false;
			case 8://bksp
			case 37://left
			case 38://up
			case 80://p
				this.xPrv(); return false;
			case 70://f
				this.toggleEffects(); return false;
			case 34://pg_dwn
				this.xNxt(true); return false;
			case 33://pg_up
				this.xPrv(true); return false;
			case 36://home
				this.loadSld(-1);return false;
			case 27://escape
				this.xHideZoom();return false;
			case 72://h
				return this.xSwitchToHtmlMode();
		}
	},
	xSwitchToHtmlMode : function(){
		var vAsw = confirm( this.xGetStr(10));
		if (vAsw){
			var vLoc = window.location;
			window.location.href = vLoc.protocol + "//" + (vLoc.hostname ? vLoc.hostname : "") + (vLoc.port ? ":" + vLoc.port : "") + (vLoc.pathname.indexOf("/")!=0 ? "/" : "") + vLoc.pathname + "?mode=html";
		}
		return false;
	},
	/** scPresMgr.xShowZoom : display the zoom window
	 * @param pRes : resource to zoom : if it is a anchor (A) then we zoom the href as a slide otherwise we clone/import the node
	 */
	xShowZoom : function(pRes){
		var vDst = null;
		var vZm = this.fZoom
		if (!vZm.fAct){
			vZm.fAct = true;
			vZm.fFrg.innerHTML = "";
			vZm.fSld.innerHTML = "";
			vZm.fSldMgr = null;
			if (pRes.nodeName.toLowerCase() == "a") {
				new scPresMgr.FadeEltTask(vZm.fFrg,0,null,null,true);
				new scPresMgr.FadeEltTask(vZm.fSld,1,null,null,true);
		
				vZm.fSldMgr = new scPresMgr.SldZoomMgr(vZm.fSld, pRes.href,function(){new scPresMgr.FadeEltTask(scPresMgr.fZoom,1,this.sFadeEltStart,this.sFadeEltEnd);});
				vZm.fSldMgr.initSld();
			} else {
				new scPresMgr.FadeEltTask(vZm.fFrg,1,null,null,true);
				new scPresMgr.FadeEltTask(vZm.fSld,0,null,null,true);
				vDst = scPresUt.importDeepNode(pRes, vZm.fFrg.ownerDocument, vZm.fFrg);
				new scPresMgr.FadeEltTask(vZm,1,this.sFadeEltStart,this.sFadeEltEnd);
			}
			this.fNav.className = this.fNav.fDefaultClass + " "+this.fSsClassPrefix+"NavZoom";
		} else {
			if (pRes.nodeName.toLowerCase() == "a") {
				// TODO finalize zoom a slide
				vZm.fSldMgr = new scPresMgr.SldZoomMgr(vZm.fSld, pRes.href);
				vZm.fSldMgr.initSld();
			} else {
				vDst = scPresUt.importDeepNode(pRes, vZm.fFrg.ownerDocument, vZm.fFrg);
				new scPresMgr.FadeEltTask(vZm.fSld,0,this.sFadeEltStart,this.sFadeEltEnd);
				new scPresMgr.FadeEltTask(vZm.fFrg,1,this.sFadeEltStart,this.sFadeEltEnd);
			}
		}
		return vDst;
	},
	/** scPresMgr.xHideZoom : hide the zoom window */
	xHideZoom : function(){
		if (this.fZoom.fAct){
			this.fZoom.fAct = false;
			new scPresMgr.FadeEltTask(this.fZoom, 0, this.sFadeEltStart, this.sHideZoomEnd);
			this.fNav.className = this.fNav.fDefaultClass;
		}
	},
	/** scPresMgr.sHideZoomEnd */
	sHideZoomEnd : function(){
		this.fElt.fSldMgr = null; 
		this.fElt.fSld.innerHTML = "";
		this.fElt.fFrg.innerHTML = "";
		if (scCoLib.isIE) this.fElt.style.backgroundColor = "";
	},
	/** scPresMgr.xGetZoomContainer : return the zoom container */
	xGetZoomContainer : function(){
		return this.fZoom.fFrg;
	},
	/** scPresMgr.xGetZoomSlide : return the zoom iframe container */
	xGetZoomSlide : function(){
		return this.fZoom.fFra;
	},
	/** scPresMgr.xResetFocus - sets the focus to the current slide. */
	xResetFocus : function() {
		if (this.fCurrSld && this.fCurrSld.fSldMgr && this.fCurrSld.fSldMgr.fFraNode){
			var vFra = this.fCurrSld.fSldMgr.fFraNode;
			var vPge = null;
			if (vFra.focus) vFra.focus();
			vPge = vFra.contentWindow.scSlideMgr.getBlocksRoot();
			if (vPge && vPge.focus) vPge.focus();
		}
	},
	/** scPresMgr.xNotifyListeners - calls all the listeners of a given type. */
	xNotifyListeners : function(pType,pRes) {
		//scCoLib.util.log("scPresMgr.xNotifyListeners: "+pType);
		var vListener = scPresMgr.fListeners[pType];
		for (i in vListener) {
			try {
				vListener[i](pRes);
			} catch(e) {scCoLib.util.logError("ERROR scPresMgr.xNotifyListeners: ",e);}
		}
	},
	/** scPresMgr.xHideTools : tool hider */
	xHideTools : function(){
		if (!this.fToolsHidden) {
			var vNewTime = new Date().getTime();
			if (vNewTime-this.fToolsShowTime >= 5000) {
				this.fToolsHidden = true;
				for (i in this.fToolElts) {
					try {
						new this.FadeEltTask(this.fToolElts[i],0,this.sFadeEltStart,this.sFadeEltEnd);
					} catch(e) {scCoLib.util.logError("ERROR scPresMgr.xHideTools: ",e);}
				}
			}
		}
	},
	/** scPresMgr.xShowTools : mouse event manager */
	xShowTools : function(){
		if (this.fToolsHidden) {
			this.fToolsHidden = false;
			for (i in this.fToolElts) {
				try {
					new this.FadeEltTask(this.fToolElts[i],1,this.sFadeEltStart,this.sFadeEltEnd);
				} catch(e) {scCoLib.util.logError("ERROR scPresMgr.xShowTools: ",e);}
			}
		}
		this.fToolsShowTime = new Date().getTime();
	},
	/** Init tous les blocks fils d'un blockRoot (du slide ou d'un container) */
	xInitBlocks : function(pParentMgr, pBlockRoot){
		var vResult = [];
		if(pBlockRoot) {
			var vNode = pBlockRoot.firstChild;
			while(vNode) {
				if(vNode.nodeType==1) {
					if(scPaLib.checkNode(scPresMgr.fContainerFilterComp, vNode)) {
						vNode.fBlkMgr = new scPresMgr.BlkContainerMgr(pParentMgr, vNode, vResult.length);
					} else {
						vNode.fBlkMgr = new scPresMgr.BlkMgr(pParentMgr, vNode, vResult.length);
					}
					vResult[vResult.length] = vNode;
					vNode.style.visibility = "hidden";
				}
				vNode = vNode.nextSibling;
			}
		}
		return vResult;
	},
	/** scPresMgr.sOnClickTocLnk : TOC item onclick event */
	sOnClickTocLnk : function(){
		scPresMgr.loadSld(this.fSldIdx, true, false);
		return false;
	},
	/** scPresMgr.sMouseMgr : mouse event manager */
	sMouseMgr : function(){
		if (scPresMgr.fToolHider) scPresMgr.xShowTools();
	},
	/** scPresMgr.sHeartbeat : run every second. */
	sHeartbeat : function(){
		if (scPresMgr.fToolHider) scPresMgr.xHideTools();
	},
	/** scPresMgr.sFadeEltStart */
	sFadeEltStart : function(){
		var vBkColor = scPresMgr.xReadStyle(this.fElt, "backgroundColor") || scPresMgr.xReadStyle(scPresMgr.fSldFra, "backgroundColor") || "white";
		if (scCoLib.isIE) this.fElt.style.backgroundColor = (vBkColor == "transparent" ? "white" : vBkColor);
	},
	/** scPresMgr.sFadeEltEnd */
	sFadeEltEnd : function(){
		if (scCoLib.isIE) this.fElt.style.backgroundColor = "";
	},
	
	/* === Utilities ========================================================== */

	/** scPresMgr.xAddElt : Add an HTML element to a parent node. */
	xAddElt : function(pName, pParent, pClassName, pNoDisplay, pHidden, pNxtSib){
		var vElt;
		if(scCoLib.isIE && pName.toLowerCase() == "iframe") {
			//BUG IE : impossible de masquer les bordures si on ajoute l'iframe via l'API DOM.
			var vFrmHolder = pParent.ownerDocument.createElement("div");
			if (pNxtSib) pParent.insertBefore(vFrmHolder,pNxtSib)
			else pParent.appendChild(vFrmHolder);
			vFrmHolder.innerHTML = "<iframe scrolling='no' frameborder='0' allowtransparency='true'></iframe>";
			vElt = vFrmHolder.firstChild;
		} else {
			vElt = pParent.ownerDocument.createElement(pName);
			if (pNxtSib) pParent.insertBefore(vElt,pNxtSib)
			else pParent.appendChild(vElt);
		}
		if (pClassName) vElt.className = pClassName;
		if (pNoDisplay) vElt.style.display = "none";
		if (pHidden) vElt.style.visibility = "hidden";
		return vElt;
	},
	/** scPresMgr.xAddEltNoDisp : Add a non displayed HTML element to a parent node. */
	xAddEltNoDisp : function(pName, pParent, pClassName) {
		var vElt = this.xAddElt(pName, pParent, pClassName, true, false);
		return vElt;
	},
	/** scPresMgr.xAddEltHidden : Add a hidden HTML element to a parent node. */
	xAddEltHidden : function(pName, pParent, pClassName) {
		var vElt = this.xAddElt(pName, pParent, pClassName, false, true);
		return vElt;
	},
	/** scPresMgr.xAddBtn : Add a HTML button to a parent node. */
	xAddBtn : function(pParent, pClassName, pCapt, pTitle, pNxtSib) {
		var vBtn = pParent.ownerDocument.createElement("a");
		vBtn.className = pClassName;
		vBtn.fName = pClassName;
		vBtn.href = "#";
		vBtn.target = "_self";
		vBtn.onclick = function() {
			scPresMgr.xBtnMgr(this);
			return false;
		}
		vBtn.setAttribute("title", pTitle);
		vBtn.innerHTML = "<span>" + pCapt + "</span>"
		if (pNxtSib) pParent.insertBefore(vBtn,pNxtSib)
		else pParent.appendChild(vBtn);
		return vBtn;
	},
	/** scPresMgr.xReadStyle : Read a style property (inline or CSS). */
	xReadStyle : function(pElt, pProp) {
		try {
			var vVal = null;
			if (pElt.style[pProp]) {
				vVal = pElt.style[pProp];
			} else if (pElt.currentStyle) {
				vVal = pElt.currentStyle[pProp];
			} else {
				var vDefaultView = pElt.ownerDocument.defaultView;
				if (vDefaultView && vDefaultView.getComputedStyle) {
					var vStyle = vDefaultView.getComputedStyle(pElt, null);
					var vProp = pProp.replace(/([A-Z])/g,"-$1").toLowerCase();
					if (vStyle[vProp]) return vStyle[vProp];
					else vVal = vStyle.getPropertyValue(vProp);
				}
			}
			return vVal.replace(/\"/g,""); //Opera returns certain values quoted (literal colors).
		} catch (e) {
			return null;
		}
	},
	/** scPresMgr.xGetEltTop : Retrun an element's absolute top (from pRoot or the top of the page).
	 * @param pElt start element.
	 * @param pRoot optional finish element, if present and is an ancestor of pElt, retun the top of pElt in relation to pRoot.
	 */
	xGetEltTop: function(pElt, pRoot) {
		var vY;
		var vRoot = pRoot || null;
		vY = scCoLib.toInt(pElt.offsetTop);
		if (pElt.offsetParent.tagName.toLowerCase() != 'body' && pElt.offsetParent.tagName.toLowerCase() != 'html' && pElt.offsetParent != vRoot) {
			vY -= pElt.offsetParent.scrollTop;
			vY += this.xGetEltTop(pElt.offsetParent, vRoot);
		}
		return vY;
	},
	/** scPresMgr.xGetEltH : Retrun an element's height. */
	xGetEltH: function(pElt) {
		return(scCoLib.toInt(pElt.style.pixelHeight || pElt.offsetHeight));
	},
	/** Set the opacity of a given node.
	 * @param pRate Variable de 0 à 1.
	 */
	xSetOpacity: function(pNode, pRate){
		if(scCoLib.isIE) pNode.filters.item("DXImageTransform.Microsoft.Alpha").opacity = pRate*100;
		else pNode.style.opacity = pRate;
	},
	/** Start the opacity of a given node.
	 * On ajoute le filtre d'opacité sur IE.
	 * On place le node en visibility: "".
	 * @param pRate 2 valeurs possibles: 0 (invisible) ou 1 (visible).
	 */
	xStartOpacityEffect: function(pNode, pRate){
		if(scCoLib.isIE) pNode.style.filter = pRate==1 ? "progid:DXImageTransform.Microsoft.Alpha(opacity=100)" : "progid:DXImageTransform.Microsoft.Alpha(opacity=0)";
		else pNode.style.opacity = pRate;
		pNode.style.visibility = "";
	},
	/** End the opacity of a given node.
	 * On supprime le filtre d'opacité sur IE (évite des bugs de refresh).
	 * On place le node en visibility: hidden.
	 * @param pRate 2 valeurs possibles: 0 (invisible) ou 1 (visible).
	 */
	xEndOpacityEffect: function(pNode, pRate){
		if(scCoLib.isIE) pNode.style.filter = "";
		else pNode.style.opacity = pRate;
		if(pRate == 0) pNode.style.visibility = "hidden";
		else pNode.style.visibility = "";
	},
	/** Reteive a localized string. */
	xGetStr: function(pStrId) {
		var vLang = navigator.userLanguage || navigator.language;
		vLang = vLang.slice(0,2).toLowerCase();
		if (this.fStrings[vLang] && this.fStrings[vLang][pStrId])	return this.fStrings[vLang][pStrId];
		else return this.fStrings["en"][pStrId];
	},

	/* === Tasks ============================================================== */
	
	/** TiLib task that swiches from the current slide to a new one. */
	fSwitchSldTask : {
		fNewSld: null,
		fStatus: null,
		fIdx: -1,
		fRateOld: [.9, .8, .7, .5, .3, .2, .1,  0,  0,  0,  0],
		fRateNew: [ 0,  0,  0,  0, .1, .2, .3, .5, .7, .8, .9],
		/** Init la task pour accéder à un nouveau slide.
		 * @param pBlock block à afficher.
		 * @param pStatus 
		 * 			"first"  : affichage du 1er step du 1er block,
		 * 			"last" : affichage du dernier step du dernier block.
		 * @param pInstantResult si true précipite le résultat instannément, sans animation.
		 */
		initTask : function(pNewSld, pStatus, pInstantResult){
			//scCoLib.util.log("fSwitchSldTask.initTask: status: "+pStatus);
			try{
			if (!scPresMgr.fEnableEffects) pInstantResult = true;
			//On annule tout autre task en cours
			scPresMgr.fSwitchStpTask.precipitateEndTask();
			scPresMgr.fSwitchBlkTask.precipitateEndTask();
			if (this.fNewSld) {
				//On annule les effets en cours pour revenir à la situation initiale.
				scPresMgr.xEndOpacityEffect(this.fNewSld, 0);
				//Enchainement rapide, plus d'animation.
				pInstantResult = true;
			}
			this.fIdx = -1;
			this.fNewSld = pNewSld;
			this.fEndTime = ( Date.now ? Date.now() : new Date().getTime() ) + 100;
			this.fStatus = pStatus;
			this.fInstantResult = pInstantResult;
			if(pInstantResult && this.fNewSld.fSldMgr.fIsLoaded) this.precipitateEndTask();
			else scTiLib.addTaskNow(this);
			}catch(e){scCoLib.util.log("ERROR fSwitchSldTask.initTask: "+e);}
		},
		execTask : function(){
			try{
			if(! this.fNewSld) return false; //precipitateEndTask() appelé entre temps.
			if(! this.fNewSld.fSldMgr.fIsLoaded) {
				//Nouvelle sld pas encore chargée, on attend...
				//Note: timeout pour sortir ? pas forcément, toute action user de navigation entrainera un cancel.
				return true;
			}
			if(this.fStatus != "") {
				//scCoLib.util.log("fSwitchSldTask.execTask: firstPass");
				//Une fois chargé, 1er passage : initialisation du block et du step demandé.
				var vNewSldMgr = this.fNewSld.fSldMgr;
				switch(this.fStatus) {
				case "first" : 
					scPresMgr.fSwitchBlkTask.initTask(vNewSldMgr.getCurrBlk(), vNewSldMgr.getFirstBlock(), "F", true);
					break;
				case "last" : 
					scPresMgr.fSwitchBlkTask.initTask(vNewSldMgr.getCurrBlk(), vNewSldMgr.getLastBlock(), "L", true);
					break;
				}
				//Début de l'animation
				//On commence à rendre visible le nouveau slide.
				scPresMgr.xStartOpacityEffect(this.fNewSld, 0);
				if(scPresMgr.fCurrSld)scPresMgr.xStartOpacityEffect(scPresMgr.fCurrSld, 1);
				this.fStatus = "";
				if(this.fInstantResult) {
					//Si on avait demandé un affichage immédiat.
					this.precipitateEndTask();
					return false;
				}
			}
			var vNow = Date.now ? Date.now() : new Date().getTime();
			while(this.fEndTime < vNow && this.fIdx < this.fRateOld.length) {
				//On saute des steps si le processor est trop lent.
				this.fIdx++;
				this.fEndTime += 100;
			}
			this.fIdx++;
			this.fEndTime += 100;
			if(this.fIdx >= this.fRateOld.length) {
				this.precipitateEndTask();
				return false;
			} else {
				if(scPresMgr.fCurrSld) scPresMgr.xSetOpacity(scPresMgr.fCurrSld, this.fRateOld[this.fIdx]);
				scPresMgr.xSetOpacity(this.fNewSld, this.fRateNew[this.fIdx]);
				return true;
			}
			}catch(e){scCoLib.util.log("ERROR fSwitchSldTask.execTask: "+e);}
		},
		/** Précipite la fin de la task en cours. */
		precipitateEndTask : function(){
			try{
			if( ! this.fNewSld) return;
			if( ! this.fNewSld.fSldMgr.fIsLoaded) throw "bug: interdit d'appeler scPresMgr.fSwitchSldTask.precipitateEndTask() lorsque le slide cible n'est pas encore chargé.";
			if(scPresMgr.fCurrSld) scPresMgr.xEndOpacityEffect(scPresMgr.fCurrSld, 0);
			if(this.fStatus != "") {
				//Le block et step n'ont pas encoré été initialisés, animation non démarrée
				var vNewSldMgr = this.fNewSld.fSldMgr;
				switch(this.fStatus) {
				case "first" : 
					scPresMgr.fSwitchBlkTask.initTask(vNewSldMgr.getCurrBlk(), vNewSldMgr.getFirstBlock(), "F", true);
					break;
				case "last" : 
					scPresMgr.fSwitchBlkTask.initTask(vNewSldMgr.getCurrBlk(), vNewSldMgr.getLastBlock(), "L", true);
					break;
				}
			}
			scPresMgr.xEndOpacityEffect(this.fNewSld, 1);
			scPresMgr.xUpdateGui();
			scPresMgr.xSetCurrSlide(this.fNewSld);
			this.fNewSld = null;
			this.fIdx = -1;
			this.fStatus = null;
			}catch(e){scCoLib.util.log("ERROR fSwitchSldTask.precipitateEndTask: "+e);}
		},
		/** Annule l'animation et retourne sur le slide en cours. */
		rollbackTask : function(){
			try{
			if( ! this.fNewSld) return;
			if(this.fStatus == "") {
				//On avait commencé à basculer...
				scPresMgr.xEndOpacityEffect(this.fNewSld, 0);
				if(scPresMgr.fCurrSld) scPresMgr.xEndOpacityEffect(scPresMgr.fCurrSld, 1);
			}
			this.fNewSld = null;
			this.fIdx = -1;
			this.fStatus = null;
			}catch(e){scCoLib.util.log("ERROR fSwitchSldTask.rollbackTask: "+e);}
		}
	},
	
	/** TiLib task that swiches from the current block to a new one. */
	fSwitchBlkTask : {
		fOldBlock: null,
		fNewBlock: null,
		fStep: 0,
		fIdx: -1,
		fRateOld: [.9, .8, .7, .5, .3, .2, .1,  0,  0,  0,  0],
		fRateNew: [ 0,  0,  0,  0, .1, .2, .3, .5, .7, .8, .9],
		/** init la task pour accéder à un nouveau block.
		 * @param pTarget Block/Step cible à afficher
		 * 			"F" : First
		 * 			"L" : Last
		 * 			"S" : Same : redraw suite à resize
		 * @param pInstantResult si true précipite le résultat instantannément, sans animation.
		 */
		initTask : function(pOldBlock, pNewBlock, pTarget, pInstantResult){
			if(!pNewBlock) return;
			//scCoLib.util.log("fSwitchBlkTask.initTask: "+pNewBlock);
			try{
			if (!scPresMgr.fEnableEffects) pInstantResult = true;
			//On précipite le chgt de steps.
			scPresMgr.fSwitchStpTask.precipitateEndTask();
			if (this.fNewBlock) {
				//On précipite notre propre task qui est en cours.
				this.precipitateEndTask();
				//Enchainement rapide, plus d'animation.
				pInstantResult = true;
			}
			//Positionnement du bloc (et de ses fils).
			pNewBlock.fBlkMgr.fixSizeAndTarget(pTarget);
			if(pOldBlock == pNewBlock) {
				//Pas d'animation si ce block est déjà le current.
				this.fOldBlock = null;
				this.fNewBlock = null;
				return;
			}
			//scCoLib.util.log("fSwitchBlkTask pInstantResult:::"+pInstantResult);
			this.fIdx = -1;
			this.fOldBlock = pOldBlock;
			this.fNewBlock = pNewBlock;
			if(pInstantResult) this.precipitateEndTask();
			else {
				//Préparation de l'animation
				this.fEndTime = ( Date.now ? Date.now() : new Date().getTime() ) + 100;
				if(this.fOldBlock) scPresMgr.xStartOpacityEffect(this.fOldBlock, 1);
				scPresMgr.xStartOpacityEffect(this.fNewBlock, 0);
				scTiLib.addTaskNow(this);
			}
			}catch(e){scCoLib.util.log("ERROR fSwitchBlkTask.initTask: "+e);}
		},
		execTask : function(){
			try{
			if(! this.fNewBlock) return false; //precipitateEndTask() appelé entre temps.
			var vNow = Date.now ? Date.now() : new Date().getTime();
			while(this.fEndTime < vNow && this.fIdx < this.fRateOld.length) {
				//On saute des steps si le processor est trop lent.
				this.fIdx++;
				this.fEndTime += 100;
			}
			this.fIdx++;
			this.fEndTime += 100;
			if(this.fIdx >= this.fRateOld.length) {
				this.precipitateEndTask();
				return false;
			} else {
				if(this.fOldBlock) scPresMgr.xSetOpacity(this.fOldBlock, this.fRateOld[this.fIdx]);
				scPresMgr.xSetOpacity(this.fNewBlock, this.fRateNew[this.fIdx]);
				return true;
			}
			}catch(e){scCoLib.util.log("ERROR fSwitchBlkTask.execTask: "+e);}
		},
		/** Précipite la fin de la task en cours. */
		precipitateEndTask : function(){
			try{
			if(!this.fNewBlock) return;
			if(this.fOldBlock) scPresMgr.xEndOpacityEffect(this.fOldBlock, 0);
			scPresMgr.xEndOpacityEffect(this.fNewBlock, 1);
			
			this.fNewBlock.fBlkMgr.fParentMgr.setCurrBlock(this.fNewBlock);
			scPresMgr.xUpdateGui();
			this.fNewBlock = null;
			this.fOldBlock = null;
			this.fIdx = -1;
			}catch(e){scCoLib.util.log("ERROR fSwitchBlkTask.precipitateEndTask: "+e);}
		}
	},
	
	/** scPresMgr.fSwitchStpTask : TiLib task that swiches from the current step to a new one. */
	fSwitchStpTask : {
		/** Paramétrage de l'animation. */
		fBlockMgr: null,
		fTargetStepIdx : -1,
		/** Objectif à atteindre en terme de déplacement du content. */
		fTargetTop : 0,
		/** Nombre de cycles restant à réaliser. */
		fCycles : -1,
		/** Mask qui va détenir le focus. */
		fNewMask : null,
		/** Mask précédent qui détenait le focus. */
		fOldMask : null,
		/** init la task pour accéder à un nouveau step.
		 * @param pTargetStep Step cible à passer en focus
		 * 			"F" : First (pInstantResult est alors forcé à true)
		 * 			"L" : Last (pInstantResult est alors forcé à true)
		 * 			"N" : Next Retourne false si pas de next, true si next trouvé.
		 * 			"P" : Previous
		 * 			"S" : Same : redraw suite à resize (pInstantResult est alors forcé à true)
		 * 					Note : le début est le même, la fin du step peut varier.
		 * @param pInstantResult si true précipite le résultat instannément, sans animation.
		 */
		initTask : function(pBlock, pTargetStep, pInstantResult){
			try{
				if (!scPresMgr.fEnableEffects) pInstantResult = true;
				if(this.fBlockMgr) {
					this.precipitateEndTask();
					//Enchainement rapide, plus d'animation.
					pInstantResult = true;
				}
				this.fCycles = -1;
				var vBlkMgr = pBlock.fBlkMgr;
				if( ! vBlkMgr.fSteps) return false;
				
				switch(pTargetStep) {
				case "F":
					this.fTargetStepIdx = 0;
					pInstantResult = true;
					break;
				case "L":
					this.fTargetStepIdx = vBlkMgr.fSteps.length-1;
					pInstantResult = true;
					break;
				case "N":
					if(vBlkMgr.fCurrStep < vBlkMgr.fSteps.length -1) this.fTargetStepIdx = vBlkMgr.fCurrStep+1;
					else return false;
					break;
				case "P":
					if(vBlkMgr.fCurrStep >0) this.fTargetStepIdx = vBlkMgr.fCurrStep-1;
					else return false;
					break;
				case "S":
					this.fTargetStepIdx = vBlkMgr.fCurrStep;
					pInstantResult = true;
					break;
				}
				this.fBlockMgr = vBlkMgr;
				//Reaffichage du mask du step en cours.
				this.fOldMask = null;
				if(vBlkMgr.fCurrStep>=0) {
					this.fOldMask = this.fBlockMgr.fMasks[vBlkMgr.fCurrStep];
					this.fOldMask.style.display="";
				}
				//Calcul de la hauteur cible
				var vAvailH = this.fBlockMgr.fBlkContent.parentNode.offsetHeight;
				this.fNewMask  = this.fBlockMgr.fMasks[this.fTargetStepIdx];
				if( ! this.fNewMask) {
				}
				this.fTargetTop = (vAvailH-this.fNewMask.offsetHeight) / 2 - this.fNewMask.offsetTop;
				//Lancement de la tache
				if(pInstantResult) this.precipitateEndTask();
				else {
					if(this.fOldMask) scPresMgr.xSetOpacity(this.fOldMask, 0);
					this.fEndTime = ( Date.now ? Date.now() : new Date().getTime() ) + 100;
					this.fCycles = Math.min(25, Math.max(10, Math.round(Math.abs(this.fBlockMgr.fBlkContent.offsetTop - this.fTargetTop)/ 10)));
					//scCoLib.util.log("this.fCycles:::"+this.fCycles);
					scTiLib.addTaskNow(this);
				}
				return true;
			}catch(e){scCoLib.util.log("ERROR fSwitchStpTask.initTask: "+e);}
		},
		execTask : function(){
			if(! this.fBlockMgr) return false; //precipitateEndTask() appelé entre temps.
			try{
			var vNow = Date.now ? Date.now() : new Date().getTime();
			while(this.fEndTime < vNow && this.fCycles >0) {
				//On saute des steps si le processor est trop lent.
				this.fCycles--;
				this.fEndTime += 100;
			}
			this.fCycles--;
			if(this.fCycles <= 0) {
				this.precipitateEndTask();
				return false;
			} else {
				this.fEndTime += 100;
				var vDeltaOpacity = Math.min(scPresMgr.fStepMaskOpacity, 4 * scPresMgr.fStepMaskOpacity / this.fCycles);
				//scCoLib.util.log("vDeltaOpacity:::"+vDeltaOpacity+" - this.fCycles::"+this.fCycles);
				if(this.fOldMask) {
					scPresMgr.xSetOpacity(this.fOldMask, vDeltaOpacity);
				}
				scPresMgr.xSetOpacity(this.fNewMask, scPresMgr.fStepMaskOpacity - vDeltaOpacity);
				var vCurrPos = this.fBlockMgr.fBlkContent.offsetTop;
				var vNewTop = vCurrPos - (2 * (vCurrPos - this.fTargetTop) / (this.fCycles+1) );
				this.fBlockMgr.fBlkContent.style.top = vNewTop+"px";
				return true;
			}
			}catch(e){scCoLib.util.log("ERROR fSwitchStpTask.execTask: "+e);}
		},
		/** Permet de précipiter la fin de la task en cours. */
		precipitateEndTask : function(){
			try{
			if(! this.fBlockMgr) return;
			//On place le content
			this.fBlockMgr.fBlkContent.style.top = this.fTargetTop+"px";
			if(this.fOldMask) {
				scPresMgr.xSetOpacity(this.fOldMask, scPresMgr.fStepMaskOpacity);
			}
			//On suppr le mask du step en cours (pour l'interactivité)
			this.fNewMask.style.display = "none";
			//On affecte le nouvel état.
			this.fBlockMgr.xSetCurrStep(this.fTargetStepIdx);
			scPresMgr.xUpdateGui();
			this.fBlockMgr = null;
			this.fTargetStepIdx = -1,
			this.fCycles = -1;
			this.fOldMask = null;
			this.fNewMask = null;
			}catch(e){scCoLib.util.log("ERROR fSwitchStpTask.precipitateEndTask: "+e);}
		}
	}
} 

/* === Managers ============================================================= */

/** -- Manager de slide standard. ------------------------------------------- */
scPresMgr.SldMgr = function(pSldNode, pIdx){
	this.fSldNode = pSldNode;
	this.fSldIdx = pIdx;
	this.fBlocksRoot = null;
	this.fIsLoaded = false;
	this.fCurrBlk = null;
	this.fFraNode = null;
}
scPresMgr.SldMgr.prototype.initSld = function(){
	this.fFraNode = scPresMgr.xAddElt("iframe", this.fSldNode, "slideFra");
	this.fFraNode.fSldMgr = this;
	this.fCurrBlk = null;
	if(scCoLib.isIE) this.fFraNode.onreadystatechange = scPresMgr.SldMgr.sOnLoadSld;
	else this.fFraNode.onload = scPresMgr.SldMgr.sOnLoadSld;
	this.fFraNode.src = scPresMgr.fSldUrls[this.fSldIdx];
}
/** Appelé sur l'event onload de la frame du slide. this == iframe du sld. */
scPresMgr.SldMgr.sOnLoadSld = function(){
	try{
		if(scCoLib.isIE && this.readyState != "complete") return;
		var vSldDoc = this.contentWindow.document;
		var vSldMgr = this.fSldMgr;
		scCoLib.util.log("scPresMgr.SldMgr.sOnLoadSld: "+vSldDoc.title);
		// Enable fSsCssName CSS on slide
		var vCss = null;
		for(var i = 0; (vCss = vSldDoc.getElementsByTagName("link")[i]); i++) {
			if(vCss.getAttribute("rel").indexOf("style") != -1 && vCss.getAttribute("title")) {
				vCss.disabled = true; //Bug IE : obliger de désactiver en premier PUIS réactiver.
				if(vCss.getAttribute("title").indexOf(scPresMgr.fSsCssName) != -1) vCss.disabled = false;
			}
		}
		
		//Load du scSlideMgr dans l'iframe.
		var vSubSlideMgr = this.contentWindow.scSlideMgr;
		vSubSlideMgr.onLoad();

		// Init Blocks
		vSldMgr.fBlocksRoot = vSubSlideMgr.getBlocksRoot();
		vSldMgr.fBlocks = scPresMgr.xInitBlocks(vSldMgr, vSldMgr.fBlocksRoot);

		// flag, load ok.
		vSldMgr.fIsLoaded = true;
		scPresMgr.xNotifyListeners("onSldSLoad", vSldMgr.fSldNode);
		vSldMgr.postOnLoad();
	} catch(e){scCoLib.util.logError("ERROR error loading slide.", e);}
}
scPresMgr.SldMgr.prototype.postOnLoad = function() {
}
scPresMgr.SldMgr.prototype.getFirstBlock = function() {
	return this.fBlocks ? this.fBlocks[0] : null;
}
scPresMgr.SldMgr.prototype.getLastBlock = function() {
	return this.fBlocks ? this.fBlocks[this.fBlocks.length-1] : null;
}
scPresMgr.SldMgr.prototype.getCurrBlk = function(){
	return this.fCurrBlk;
}
/** Navigue à l'intérieur du slide (ie dans les blocks et steps) pour avancer d'un cran. */
scPresMgr.SldMgr.prototype.goToNxt = function(){
	//On précipite les animations en cours.
	scPresMgr.fSwitchBlkTask.precipitateEndTask();
	var vNxt;
	if(this.fCurrBlk) {
		if(this.fCurrBlk.fBlkMgr.goToNxt()) return true;
		vNxt = this.fBlocks[this.fCurrBlk.fBlkMgr.fBlkIdx+1];
	} else {
		vNxt = this.fBlocks[0];
	}
	if(vNxt) {
		scPresMgr.fSwitchBlkTask.initTask(this.fCurrBlk, vNxt, "F", false);
		return true;
	}
	return false;
}
/** Navigue à l'intérieur du slide (ie dans les blocks et steps) pour reculer d'un cran. */
scPresMgr.SldMgr.prototype.goToPrv = function(){
	//On précipite les animations en cours.
	scPresMgr.fSwitchBlkTask.precipitateEndTask();
	var vNxt;
	if(this.fCurrBlk) {
		if(this.fCurrBlk.fBlkMgr.goToPrv()) return true;
		vNxt = this.fBlocks[this.fCurrBlk.fBlkMgr.fBlkIdx-1];
	} else {
		vNxt = this.fBlocks[this.fBlocks.length-1];
	}
	if(vNxt) {
		scPresMgr.fSwitchBlkTask.initTask(this.fCurrBlk, vNxt, "L", false);
		return true;
	}
	return false;
}
scPresMgr.SldMgr.prototype.hasNxt = function(){
	if(this.fCurrBlk) {
		if(this.fCurrBlk.fBlkMgr.hasNxt()) return true;
		return this.fCurrBlk.fBlkMgr.fBlkIdx < this.fBlocks.length - 1;
	} else return this.fBlocks.length > 1;
}
scPresMgr.SldMgr.prototype.hasPrv = function(){
	if(this.fCurrBlk) {
		if(this.fCurrBlk.fBlkMgr.hasPrv()) return true;
		return this.fCurrBlk.fBlkMgr.fBlkIdx > 0;
	} else return this.fBlocks.length > 1;
}
scPresMgr.SldMgr.prototype.redrawSld = function(){
	//On précipite les animations en cours.
	scPresMgr.fSwitchBlkTask.precipitateEndTask();
	//On redessine le block en cours et invalide les autres blocks.
	for(var i = 0, vLen = this.fBlocks.length; i < vLen; i++) {
		var vBlk = this.fBlocks[i];
		if(vBlk === this.fCurrBlk) {
			vBlk.fBlkMgr.redrawBlk();
		} else {
			vBlk.fBlkMgr.invalidateSize();
		}
	}
}
scPresMgr.SldMgr.prototype.invalidateSize = function(){
	for(var i = 0; i < this.fBlocks.length; i++) {
		this.fBlocks[i].fBlkMgr.invalidateSize();
	}
}
scPresMgr.SldMgr.prototype.getAvailHeight = function(){
	return this.fFraNode.contentWindow.scSlideMgr.getBlocksAvailHeight();
}
scPresMgr.SldMgr.prototype.getAvailWidth = function(){
	return this.fFraNode.contentWindow.scSlideMgr.getBlocksAvailWidth();
}
scPresMgr.SldMgr.prototype.keyMgr = function(pCharCode){
	return scPresMgr.xKeyMgr(pCharCode);
}
scPresMgr.SldMgr.prototype.mouseMgr = function(){
	return scPresMgr.sMouseMgr();
}
scPresMgr.SldMgr.prototype.showZoom = function(pRes){
	return scPresMgr.xShowZoom(pRes);
}
scPresMgr.SldMgr.prototype.getZoomContainer = function(pRes){
	return scPresMgr.xGetZoomContainer();
}
scPresMgr.SldMgr.prototype.getZoomSlide = function(pRes){
	return scPresMgr.xGetZoomSlide();
}
scPresMgr.SldMgr.prototype.setCurrBlock = function(pNewBlock){
	this.fCurrBlk = pNewBlock;
}
scPresMgr.SldMgr.prototype.isFirstSld = function(){
	return this.fSldIdx == -1;
}
scPresMgr.SldMgr.prototype.isZoomSld = function(){
	return this.fSldIdx == -2;
}
scPresMgr.SldMgr.prototype.isLastSld = function(){
	return this.fSldIdx == scPresMgr.fSldUrls.length - 1;
}

/** -- Manager du slide de l'accueil : pas d'iframe. ------------------------ */
scPresMgr.SldHomeMgr = function(pSldNode){
	this.fSldNode = pSldNode;
	this.fSldIdx = -1;
	this.fIsLoaded = false;
	this.fCurrBlk = null;
}
scPresMgr.SldHomeMgr.prototype = new scPresMgr.SldMgr();
scPresMgr.SldHomeMgr.prototype.initSld = function(){
	this.fCurrBlk = null;
	//Load 
	scSlideMgr.onLoad();
	// Init Blocks
	this.fBlocksRoot = scSlideMgr.getBlocksRoot();
	this.fBlocks = scPresMgr.xInitBlocks(this, this.fBlocksRoot);
	// flag, load ok.
	this.fIsLoaded = true;
}
scPresMgr.SldHomeMgr.prototype.redrawSld = function() {
	//On précipite les animations en cours.
	scPresMgr.fSwitchBlkTask.precipitateEndTask();
	//On redessine le block en cours.
	if(this.fCurrBlk) this.fCurrBlk.fBlkMgr.redrawBlk();
}
scPresMgr.SldHomeMgr.prototype.getAvailHeight = function(){
	return this.fBlocksRoot.offsetHeight;
}
scPresMgr.SldHomeMgr.prototype.getAvailWidth = function(){
	return this.fBlocksRoot.offsetWidth;
}

/** -- Manager de slide de zoom. -------------------------------------------- */
scPresMgr.SldZoomMgr = function(pSldNode, pUrl, pPostLoadFunc){
	this.fSldNode = pSldNode;
	this.fSldIdx = -2;
	this.fPostLoadFunc = pPostLoadFunc || function(){};
	this.fSldUrl = pUrl;
	this.fIsLoaded = false;
	this.fCurrBlk = null;
}
scPresMgr.SldZoomMgr.prototype = new scPresMgr.SldMgr();
scPresMgr.SldZoomMgr.prototype.initSld = function(){
	this.fFraNode = scPresMgr.xAddElt("iframe", this.fSldNode, "slideFra");
	this.fFraNode.fSldMgr = this;
	this.fCurrBlk = null;
	if(scCoLib.isIE) this.fFraNode.onreadystatechange = scPresMgr.SldMgr.sOnLoadSld;
	else this.fFraNode.onload = scPresMgr.SldMgr.sOnLoadSld;
	this.fFraNode.src = this.fSldUrl;
}
scPresMgr.SldZoomMgr.prototype.postOnLoad = function() {
	scPresMgr.fSwitchBlkTask.initTask(this.fCurrBlk, this.getFirstBlock(), "F", true);
	this.fPostLoadFunc();
}

/** -- Manager de blocks dans un slide. ------------------------------------- */
scPresMgr.BlkMgr = function(pParentMgr, pBlkNode, pBlkIdx){
	this.fParentMgr = pParentMgr;
	this.fBlkNode = pBlkNode;
	this.fBlkIdx = pBlkIdx;
	this.fBlkContent = scPaLib.findNode(scPresMgr.fBlkCoPathComp, this.fBlkNode);
//	this.fBlkContent = pBlkNode.firstChild;
//	while(this.fBlkContent && this.fBlkContent.nodeType != 1) this.fBlkContent = this.fBlkContent.nextSibling;
	/** Etapes à afficher. */
	this.fSteps = null;
	/** n° de l'étape en cours. */
	this.fCurrStep = -1;

	// Recherche de tous les points de coupe possibles - on ne prend pas en compte ceux qui sont dans une table.
	var vSteps = scPaLib.findNodes("des:"+scPresMgr.fCutableFilter, this.fBlkContent);
	this.fAllSteps = [];
	for (var i in vSteps) if(!scPaLib.findNode("anc:"+scPresMgr.fUncutableFilter,vSteps[i])) this.fAllSteps.push(vSteps[i]);
	if (this.fAllSteps.length < 2) this.fAllSteps = null;
	
	// Calcul des étapes en fonction de l'espace disponible.
	this.xPlanSteps();

	// Centre le block dans son container
	this.xCenterBlock();

	this.fNeedResize = false;
}
/** Redraw suite à un resize. Le block est en cours... */
scPresMgr.BlkMgr.prototype.redrawBlk = function(){
	//On précipite les animations en cours sur le steps.
	scPresMgr.fSwitchStpTask.precipitateEndTask();
	//On mémorise la situation
	var vPreviousFocusNode = this.fSteps ? this.fSteps[this.fCurrStep] : null;
	//On efface tout positionnement précédent
	this.xResetSizeStepsAndBlocks();
	//On replanifie les steps
	this.xPlanSteps();
	//On recherche la step à afficher
	if(this.fSteps) {
		//Par défaut, step 0.
		this.fCurrStep = 0;
		if(vPreviousFocusNode) {
			//Recherche directe dans le tableau des nouvelles steps
			for(var i=this.fSteps.length-1; i >= 0; i--) {
				if(vPreviousFocusNode==this.fSteps[i]) {
					this.fCurrStep = i;
					break;
				}
			}
			if(this.fCurrStep==-1) {
				//Pas trouvé, on recherche vPreviousFocusNode dans toutes les steps
				var vFirstFocusIdx = -1;
				for(var i=this.fAllSteps.length-1; i >= 0; i--) {
					if(vPreviousFocusNode==this.fAllSteps[i]) {
						vFirstFocusIdx = i;
						break;
					}
				}
				if(vFirstFocusIdx != -1) {
					search:
					for(var i=vFirstFocusIdx-1; i >= 0; i--) {
						var vStep = this.fAllSteps[i];
						for(var j=this.fSteps.length-1; j >= 0; j--) {
							if(vStep==this.fSteps[j]) {
								//On a trouvé la l'étape qui contient vPreviousFocusNode
								this.fCurrStep = j;
								break search;
							}
						}
					}
				}
			}
		}
	} else this.fCurrStep = -1;
	// Centre le block dans son container
	this.xCenterBlock();
	// Resize ok
	this.fNeedResize = false;
	//On resize et redessine le block
	scPresMgr.fSwitchBlkTask.initTask(this.fBlkNode, this.fBlkNode, "S", true);
}
/** Invalide la taille du block. Le block n'est pas actif. */
scPresMgr.BlkMgr.prototype.invalidateSize = function(){
	this.fNeedResize = true;
}
/** Calcul les dimensions du block si nécessaire et redessine les steps du block terminal(appelé par la BlkTask)
 * 
 * @param pTarget Block/Step cible à afficher
 * 			"F" : First
 * 			"L" : Last
 * 			"S" : Same : redraw suite à resize
 */
scPresMgr.BlkMgr.prototype.fixSizeAndTarget = function(pTarget){
	if(this.fNeedResize) {
		// Reset
		this.xResetSizeStepsAndBlocks();
		// Calcul des étapes en fonction de l'espace disponible.
		this.xPlanSteps();
		// Centre le block dans son container
		this.xCenterBlock();
		// Flag ok
		this.fNeedResize = false;
	}
	//Redraw sans animation des steps du block.
	scPresMgr.fSwitchStpTask.initTask(this.fBlkNode, pTarget, true);
}
/** Step suivant */
scPresMgr.BlkMgr.prototype.goToNxt = function(){
	return scPresMgr.fSwitchStpTask.initTask(this.fBlkNode, "N", false);
}
/** Step précédent */
scPresMgr.BlkMgr.prototype.goToPrv = function(){
	return scPresMgr.fSwitchStpTask.initTask(this.fBlkNode, "P", false);
}
scPresMgr.BlkMgr.prototype.hasNxt = function(){
	if (this.fSteps) return this.fCurrStep < this.fSteps.length -1;
	else return false;
}
scPresMgr.BlkMgr.prototype.hasPrv = function(){
	return this.fCurrStep > 0;
}
/** Reset les step à afficher. */
scPresMgr.BlkMgr.prototype.xResetSizeStepsAndBlocks = function(){
	//On reset un éventuel resize forcé précédent.
	var vBlkNodeStyle = this.fBlkNode.style;
	//On reset l'ajustement éventuel de la police.
	vBlkNodeStyle.fontSize="";
	//On reset le positionnement du block (restrictions en largeur du block).
	vBlkNodeStyle.top = "0px";
	vBlkNodeStyle.left = "0px";
	vBlkNodeStyle.right = "0px";
	if(this.fSteps) {
		var vParentContentStyle = this.fBlkContent.parentNode.style;
		//On reset un précédent calcul des steps
		//vParentContentStyle.overflow= "none";
		vParentContentStyle.height= "";
		var vBlkContentStyle = this.fBlkContent.style;
		vBlkContentStyle.position = "relative";
		vBlkContentStyle.top = "0px";
		this.fSteps = null;
	}
}
/** Planifie les step à afficher. */
scPresMgr.BlkMgr.prototype.xPlanSteps = function(){
	this.fSteps = null;
	var vBlkContentStyle = this.fBlkContent.style;
	var vParentContentStyle = this.fBlkContent.parentNode.style;
	var vAvailH = this.fParentMgr.getAvailHeight();
	var vRealH = this.fBlkNode.offsetHeight;
	if(vRealH < vAvailH) {
		//Pas besoin de créer des steps
		this.xRemoveMasks();
		return;
	}
	//On tente un léger resize de la font
	vBlkContentStyle.fontSize="85%";
	vRealH = this.fBlkNode.offsetHeight;
	if(vRealH < vAvailH) {
		//Pas besoin de créer des steps
		this.xRemoveMasks();
		return;
	}
	//On crée les steps (si on a des marqueurs)
	if(! this.fAllSteps || this.fAllSteps.length==0) return;
	//On fige les dimensions du content du block
	var vContentH = Math.max(20, vAvailH - scPresMgr.xGetEltTop(this.fBlkNode,this.fParentMgr.fBlocksRoot) - (this.fBlkNode.offsetHeight - this.fBlkContent.offsetHeight));
	vBlkContentStyle.position = "absolute";
	vBlkContentStyle.width = "100%";
	vBlkContentStyle.top = "0px";
	vParentContentStyle.overflow = "hidden";
	vParentContentStyle.height = vContentH+"px";
	//Le 1er step est nécessairement le content du block
	this.fSteps = [this.fBlkContent];
	//Sélection des steps
	var vPreviousTop = 0;
	var vPreviousAllStepsIdx = -1;
	var vTotalSteps = this.fAllSteps.length;
	//Fonction de calcul du score de chaque point de coupe
	var vBlkContent = this.fBlkContent;
	function score(pNode){
		var vTop = pNode.offsetTop;
		var vH = vTop - vPreviousTop;
		if(vH > vContentH) return -1;
		//Malus en fonction de la profondeur de cuttable au dessus
		var vP = pNode.parentNode;
		while(vP != vBlkContent) {
			if(scPaLib.checkNode(scPresMgr.fCutableFilterComp, vP)) vH *= .7;
			vP = vP.parentNode;
		}
		//Malus si le dernier step devient plus petit que celui-là
		if( vRealH - vTop < vTop - vPreviousTop) vH -= (vContentH - (vRealH - vTop))/2;
		return 1000000 + vH;
	}
	do {
		var vBestScore = -1;
		var vBestIdx = -1;
		for(var i = vPreviousAllStepsIdx+1; i<vTotalSteps; i++) {
			var vScore = score(this.fAllSteps[i]);
			if(vScore > vBestScore) {
				vBestScore = vScore;
				vBestIdx = i;
			} else if(vScore<0) {
				//si <0 : hors zone visible.
				break;
			}
		}
		if(vBestIdx>=0) {
			var vNode = this.fAllSteps[vBestIdx];
			this.fSteps[this.fSteps.length] = this.fAllSteps[vBestIdx];
			vPreviousTop = vNode.offsetTop;
			vPreviousAllStepsIdx = vBestIdx;
		}
	} while(vBestIdx>=0 && (vRealH-vPreviousTop > vContentH));
	//On crée le tableau des masks
	if(! this.fMasks) this.fMasks = [];
	//On crée autant de masks que de steps
	var vBgColor = scPresMgr.xReadStyle(this.fBlkContent, "backgroundColor") || "white";
	if(vBgColor=="transparent" || vBgColor.search(/rgba\([0-9 ]*,[0-9 ]*,[0-9 ]*,\s?0\s?\)/)>-1) vBgColor = scPresMgr.xReadStyle(this.fBlkNode, "backgroundColor") || "white";
	if(vBgColor=="transparent" || vBgColor.search(/rgba\([0-9 ]*,[0-9 ]*,[0-9 ]*,\s?0\s?\)/)>-1) vBgColor = "white";
	for(var i = this.fMasks.length; i < this.fSteps.length; i++) {
		this.fMasks[i] = this.xCreateMask(vBgColor);
	}
	//On purge les masks en trop
	for(var i = this.fSteps.length; i < this.fMasks.length; i++) {
		this.fBlkContent.removeChild(this.fMasks[i]);
	}
	this.fMasks.length = this.fSteps.length;
	
	//On place les masks.
	var vPreviousBottom = 0;
	var vLastIdx = this.fSteps.length - 1;
	for(var i = 0; i <= vLastIdx; i++) {
		var vMask = this.fMasks[i];
		var vNextTop = i < vLastIdx ? this.fSteps[i+1].offsetTop : this.fBlkContent.offsetHeight;
		vMask.style.top = vPreviousBottom+"px";
		vMask.style.height = (vNextTop - vPreviousBottom)+"px";
		vMask.style.display="";
		scPresMgr.xSetOpacity(vMask, scPresMgr.fStepMaskOpacity);
		vPreviousBottom = vNextTop;
	}
}
scPresMgr.BlkMgr.prototype.xCenterBlock = function() {
	var vParentMgr = this.fParentMgr;
	var vAvailH = vParentMgr.getAvailHeight();
	var vW = this.fBlkNode.offsetWidth;
	var vH = this.fBlkNode.offsetHeight;
	var vMargin = 0;
	if(vH < vAvailH) {
		var vAvailW = vParentMgr.getAvailWidth();
		if(vAvailW > scPresMgr.fBlockMinWidth) {
			vMargin = Math.round(Math.min( vAvailW * scPresMgr.fBlockMaxMargin * (1 - vH / vAvailH), (vAvailW - scPresMgr.fBlockMinWidth)/2) );
		}
	}
	//console.log("vMargin:::"+vMargin);
	this.fBlkNode.style.left = vMargin + "px";
	this.fBlkNode.style.right = vMargin + "px" ;
	vAvailH = vParentMgr.getAvailHeight();
	vH = this.fBlkNode.offsetHeight;
	if(vH < vAvailH) {
		this.fBlkNode.style.top = Math.round( (vAvailH-vH) * scPresMgr.fBlockTopSpace)+"px";
	}
}
/** affecte la step stabilisée (appelé par fSwitchStpTask) */
scPresMgr.BlkMgr.prototype.xSetCurrStep = function(pCurrStep){
	this.fCurrStep = pCurrStep;
}
/** supprime tous les masks d'un block. s*/
scPresMgr.BlkMgr.prototype.xRemoveMasks = function(){
	if(!this.fMasks) return;
	for(var i = this.fMasks.length-1; i >=0; i--) {
		this.fBlkContent.removeChild(this.fMasks[i]);
	}
	this.fMasks = null;
}
/** Cree un mask pour un step d'un slide. */
scPresMgr.BlkMgr.prototype.xCreateMask = function(pBgColor) {
	var vMask = scPresMgr.xAddElt("div", this.fBlkContent, "ssMask");
	vMask.style.position = "absolute";
	vMask.style.left = "0px";
	vMask.style.width = "100%";
	vMask.style.height = "0px";
	vMask.style.backgroundColor = pBgColor;
	if (scCoLib.isIE) vMask.style.filter = "progid:DXImageTransform.Microsoft.Alpha(opacity=100)";
	return vMask;
}	

/** -- Manager de blocks de type container  de sous-blocks. ----------------- */
scPresMgr.BlkContainerMgr = function(pParentMgr, pBlkContainerNode, pBlkIdx){
	this.fParentMgr = pParentMgr;
	this.fBlkNode = pBlkContainerNode;
	this.fBlkIdx = pBlkIdx;
	// Racine des blocks du container.
	this.fBlocksRoot = scPaLib.findNode(scPresMgr.fContainerBlocksRootPathComp, this.fBlkNode);
	//Fixe les dim du container
	this.xFixContainerSize();
	//init les blocks de ce container
	this.fSubBlocks = scPresMgr.xInitBlocks(this, this.fBlocksRoot);
	// Sous-bloc en cours.
	this.fCurrSubBlk = null;
}
scPresMgr.BlkContainerMgr.prototype.getFirstBlock = function() {
	return this.fSubBlocks ? this.fSubBlocks[0] : null;
}
scPresMgr.BlkContainerMgr.prototype.getLastBlock = function() {
	return this.fSubBlocks ? this.fSubBlocks[this.fSubBlocks.length-1] : null;
}
scPresMgr.BlkContainerMgr.prototype.getCurrBlk = function(){
	return this.fCurrSubBlk;
}
scPresMgr.BlkContainerMgr.prototype.setCurrBlock = function(pNewBlock){
	this.fCurrSubBlk = pNewBlock;
}
/** Navigue à l'intérieur du container pour avancer d'un cran. */
scPresMgr.BlkContainerMgr.prototype.goToNxt = function(){
	var vNxt;
	if(this.fCurrSubBlk) {
		if(this.fCurrSubBlk.fBlkMgr.goToNxt()) return true;
		vNxt = this.fSubBlocks[this.fCurrSubBlk.fBlkMgr.fBlkIdx+1];
	} else {
		vNxt = this.fSubBlocks[0];
	}
	if(vNxt) {
		scPresMgr.fSwitchBlkTask.initTask(this.fCurrSubBlk, vNxt, "F", false);
		return true;
	}
	return false;
}
/** Navigue à l'intérieur du container pour reculer d'un cran. */
scPresMgr.BlkContainerMgr.prototype.goToPrv = function(){
	var vNxt;
	if(this.fCurrSubBlk) {
		if(this.fCurrSubBlk.fBlkMgr.goToPrv()) return true;
		vNxt = this.fSubBlocks[this.fCurrSubBlk.fBlkMgr.fBlkIdx-1];
	} else {
		vNxt = this.fSubBlocks[this.fSubBlocks.length-1];
	}
	if(vNxt) {
		scPresMgr.fSwitchBlkTask.initTask(this.fCurrSubBlk, vNxt, "L", false);
		return true;
	}
	return false;
}
scPresMgr.BlkContainerMgr.prototype.hasNxt = function(){
	if(this.fCurrSubBlk) {
		if(this.fCurrSubBlk.fBlkMgr.hasNxt()) return true;
		return this.fCurrSubBlk.fBlkMgr.fBlkIdx < this.fSubBlocks.length - 1;
	} else return this.fSubBlocks.length > 1;
}
scPresMgr.BlkContainerMgr.prototype.hasPrv = function(){
	if(this.fCurrSubBlk) {
		if(this.fCurrSubBlk.fBlkMgr.hasPrv()) return true;
		return this.fCurrSubBlk.fBlkMgr.fBlkIdx > 0;
	} else return this.fSubBlocks.length > 1;
}
scPresMgr.BlkContainerMgr.prototype.redrawBlk = function(){
	//On précipite les animations en cours.
	scPresMgr.fSwitchBlkTask.precipitateEndTask();
	//On redessine le block en cours et invalide les autres blocks.
	for(var i = 0, vLen = this.fSubBlocks.length; i < vLen; i++) {
		var vBlk = this.fSubBlocks[i];
		if(vBlk === this.fCurrSubBlk) {
			vBlk.fBlkMgr.redrawBlk();
		} else {
			vBlk.fBlkMgr.invalidateSize();
		}
	}
}
/** Invalide la taille du block. Le block n'est pas actif. */
scPresMgr.BlkContainerMgr.prototype.invalidateSize = function(){
	for(var i = 0, vLen = this.fSubBlocks.length; i < vLen; i++) {
		this.fSubBlocks[i].fBlkMgr.invalidateSize();
	}
	this.fNeedResize = true;
}
/** Calcul les dimensions du block si nécessaire et redessine es sous-blocs et les steps du block terminal(appelé par la BlkTask)
 * 
 * @param pTarget Block/Step cible à afficher
 * 			"F" : First
 * 			"L" : Last
 * 			"S" : Same : redraw suite à resize
 */
scPresMgr.BlkContainerMgr.prototype.fixSizeAndTarget = function(pTarget){
	if(this.fNeedResize) this.xFixContainerSize();
	var vNewSubBlk;
	switch(pTarget) {
		case "F" : {
			vNewSubBlk = this.getFirstBlock();
			break;
		}
		case "L" : {
			vNewSubBlk = this.getLastBlock();
			break;
		}
		case "S" : {
			vNewSubBlk = this.fCurrSubBlk;
			break;
		}
	}
	if(vNewSubBlk) {
		//Gestion de l'affichage instantanné des sous-blocs.
		if(this.fCurrSubBlk && this.fCurrSubBlk !== vNewSubBlk) {
			scPresMgr.xEndOpacityEffect(this.fCurrSubBlk, 0);
		}
		scPresMgr.xEndOpacityEffect(vNewSubBlk, 1);
		vNewSubBlk.fBlkMgr.fixSizeAndTarget(pTarget);
		this.setCurrBlock(vNewSubBlk);
	}
}
/** Hauteur disponible pour les fils de ce container. */
scPresMgr.BlkContainerMgr.prototype.getAvailHeight = function(){
	return this.fBlocksRoot.offsetHeight;
}
/** Largeur disponible pour les fils de ce container. */
scPresMgr.BlkContainerMgr.prototype.getAvailWidth = function(){
	return this.fBlocksRoot.offsetWidth;
}
/** Fixe la taille du container dans son contexte parent. */
scPresMgr.BlkContainerMgr.prototype.xFixContainerSize = function(){
	var vBodyH = this.fParentMgr.getAvailHeight();
	var vTop = this.fBlocksRoot.offsetTop;
	//scCoLib.util.log("vBodyH: "+vBodyH+" - vTop"+vTop);
	this.fBlocksRoot.style.height = Math.max(50, vBodyH - vTop)+"px";
	this.fNeedResize = false;
}

/** scPresMgr.FadeEltTask : TiLib task that fades a given element in or out.
 * @param pElt element to fade.
 * @param pDir fade direction : 0=out, 1=in.
 * @param pStartFunc optionnal function that will be executed at the start of the task.
 * @param pEndFunc optionnal function that will be executed at the end of the task.
 * @param pInstant optionnal parameter if true no animation.
 */
scPresMgr.FadeEltTask = function(pElt,pDir,pStartFunc,pEndFunc,pInstant){
	this.fRate = new Array();
	this.fRate[0] = [.9, .85, .8, .7, .6, .5, .4, .3, .2, .15, .1];
	this.fRate[1] = [.1, .15, .2, .3, .4, .5, .6, .7, .8, .85, .9];
	try{
		this.fElt = pElt;
		this.fDir = (pDir >= 1 ? 1 : 0);
		this.fStartFunc = pStartFunc || function(){};
		this.fEndFunc = pEndFunc || function(){};
		if (pInstant || !scPresMgr.fEnableEffects) {
			this.terminate();
			return;
		}
		if (this.fElt.fFadeTask) {
			this.fElt.fFadeTask.changeDir(this.fDir);
		} else {
			this.fStartFunc();
			scPresMgr.xStartOpacityEffect(this.fElt, 1-this.fDir);
			this.fEndTime = ( Date.now  ? Date.now() : new Date().getTime() ) + 100;
			this.fIdx = -1;
			this.fElt.fFadeTask = this;
			scTiLib.addTaskNow(this);
		}
	}catch(e){scCoLib.util.log("ERROR scPresMgr.FadeEltTask: "+e);}
}
scPresMgr.FadeEltTask.prototype.execTask = function(){
	while(this.fEndTime < (Date.now ? Date.now() : new Date().getTime()) && this.fIdx < this.fRate[this.fDir].length) {
		this.fIdx++;
		this.fEndTime += 100;
	}
	this.fIdx++;
	this.fEndTime += 100;
	if(this.fIdx >= this.fRate[this.fDir].length) {
		scPresMgr.xEndOpacityEffect(this.fElt, this.fDir);
		this.fEndFunc();
		this.fElt.fFadeTask = null;
		return false;
	}
	scPresMgr.xSetOpacity(this.fElt, this.fRate[this.fDir][this.fIdx]);
	return true;
}
scPresMgr.FadeEltTask.prototype.changeDir = function(pDir){
	var vDir = (pDir >= 1 ? 1 : 0)
	if (vDir != this.fDir) this.fIdx = this.fRate[this.fDir].length - this.fIdx - 1;
	this.fDir = vDir;
}
scPresMgr.FadeEltTask.prototype.terminate = function(){
	this.fIdx = this.fRate[this.fDir].length;
	this.execTask();
}

/** scSlideMgr : scSlideMgr pour le homeSlide. */
var scSlideMgr = {
	fDefaultFontSize : 22, // fontSize in pixels
	fBlocksPath : "des:.ssBkRoot",
	init : function() {
	},
	setBlocksPath : function(pBlocksPath) {
		this.fBlocksPath = pBlocksPath;
	},
	/** Appelé par scPresMgr, pas par le framework standardscOnLoads. */
	onLoad : function() {
		this.fBlocksRoot = scPaLib.findNode(this.fBlocksPath, scPresMgr.fSldHome);
		this.xFixRatioNormalScreen();
		this.xFixBlocksRootSize();
		scSiLib.addRule(document.body, this);
	},
	getBlocksRoot : function(){
		return this.fBlocksRoot;
	},
	getBlocksAvailHeight : function(){
		return this.fBlocksRoot.offsetHeight;
	},
	getBlocksAvailWidth : function(){
		return this.fBlocksRoot.offsetWidth;
	},
	/** Api scSiLib. */
	onResizedAnc : function(pOwnerNode, pEvent) {
		//Resize du slide.
		if(pEvent.phase==1) {
			this.xFixRatioNormalScreen();
			this.xFixBlocksRootSize();
		} else {
			scPresMgr.redrawSlideZone();
		}
	},
	/** Api scSiLib. */
	onResizedDes : function(pOwnerNode, pEvent) {
	},
	/** Api scSiLib. */
	ruleSortKey : "AA",

	/* === Private ============================================================ */
	xFixBlocksRootSize : function(){
		var vBodyH = this.xBodyHeight();
		var vTop = this.fBlocksRoot.offsetTop;
		this.fBlocksRoot.style.height = Math.max(50, vBodyH - vTop)+"px";
	},
	xBodyHeight : function() {
		return scPresMgr.fSldFra.offsetHeight;
	},
	xBodyWidth : function() {
		return scPresMgr.fSldFra.offsetWidth;
	},
	xFixRatioNormalScreen : function(){
		this.fRatioNormalScreen = Math.sqrt(this.xBodyHeight() / 600 * this.xBodyWidth() / 800);
		scPresMgr.fSldHome.style.fontSize = Math.round(this.fRatioNormalScreen * this.fDefaultFontSize)+"px";
	}
}

/** scPresUt : Browser dependant utility toolbox. */
var scPresUt = {};
if(scCoLib.isIE) {
	scPresUt.serialiseObjJs = function(pObj){
		var vBuf="";
		if(pObj) for (var vKey in pObj){
			var vObj = pObj[vKey];
			if(vObj) {
				switch(typeof vObj) {
					case "object" : 
						vBuf+=( vBuf!="" ? ",'" : "'")+vKey+"':"+this.serialiseObjJs(vObj);
						break;
					case "string" : 
						vBuf+=( vBuf!="" ? ",'" : "'")+vKey+"':'"+vObj.replace(/[\t\n\r'"]/g, this.escapeJs)+"'";
						break;
					case "number" :
					case "boolean" :
						vBuf+=( vBuf!="" ? ",'" : "'")+vKey+"':"+vObj;
						break;
					default : 
						vBuf+=( vBuf!="" ? ",'" : "'")+vKey+"':'"+vObj.toString().replace(/[\t\n\r'"]/g, this.escapeJs)+"'";
						break;
				}
			}
		}
		return "{"+vBuf+"}";
	};
	scPresUt.importElt = function(pEltSrc, pDocDst, pParentDst, pChildDst) {
		try {
			var vWrapper;
			var vDeep = false;
			switch(pEltSrc.nodeName) {
				case "TBODY" :
				case "TR" :
					var vRes = pDocDst.createElement(pEltSrc.nodeName);
					//TODO gestion attributs : pEltSrc.cloneNode(false).outerHTML + regEpx pour extraire les attr  ou parseXml ou gestion en dur
					if(pParentDst) pParentDst.appendChild(vRes);
					if(pChildDst) vRes.appendChild(pChildDst);
					return vRes;
				case "TD" :
					vWrapper = pDocDst.createElement("TR");
					break;
				case "OPTION" :
					vWrapper = pDocDst.createElement("SELECT");
					break;
				case "OBJECT" :
					//Cas particulier : on clone Objet et param d'un coup.
					vWrapper = pDocDst.createElement("DIV");
					vDeep = true;
					break;
				case "PARAM" :
					//Cas particulir déjà traité/
					return null;
				default :
					vWrapper = pDocDst.createElement("DIV");
			}
			vWrapper.innerHTML = vDeep ? pEltSrc.outerHTML : pEltSrc.cloneNode(false).outerHTML;
			var vRes = vWrapper.firstChild;
			while(vRes && vRes.nodeType!=1) vRes = vRes.nextSibling;
			if(vRes) {
				if(pParentDst) pParentDst.appendChild(vRes);
				if(pChildDst) vRes.appendChild(pChildDst);
			}
			return vRes;
		} catch(e){}
	}
	scPresUt.importNode = function(pEltSrc, pDocDst, pParentDst) {
		switch(pEltSrc.nodeType){
			case 1 : 
				return this.importElt(pEltSrc, pDocDst, pParentDst);
			case 3 : 
				var vElt = pDocDst.createTextNode(pEltSrc.nodeValue);
				if(pParentDst) pParentDst.appendChild(vElt);
				return vElt;
		}
	}
	scPresUt.importDeepNode = function(pEltSrc, pDocDst, pParentDst) {
		switch(pEltSrc.nodeType){
			case 1 : 
				var vElt = this.importElt(pEltSrc, pDocDst, pParentDst);
				for(var vCh = pEltSrc.firstChild; vCh; vCh = vCh.nextSibling) this.importDeepNode(vCh, pDocDst, vElt);
				return vElt;
			case 3 : 
				var vElt = pDocDst.createTextNode(pEltSrc.nodeValue);
				if(pParentDst) pParentDst.appendChild(vElt);
				return vElt;
		}
	}
} else {
	scPresUt.serialiseObjJs = function(pObj){return (pObj) ? pObj.toSource() : null;}
	scPresUt.importElt = function(pEltSrc, pDocDst, pParentDst, pChildDst) {
		var vRes = pDocDst.importNode(pEltSrc, false);
		if(pParentDst) pParentDst.appendChild(vRes);
		if(pChildDst) vRes.appendChild(pChildDst);
		return vRes;
	}
	scPresUt.importNode = scPresUt.importElt;
	scPresUt.importDeepNode = function(pEltSrc, pDocDst, pParentDst) {
		var vElt = pDocDst.importNode(pEltSrc, true);
		if(pParentDst) pParentDst.appendChild(vElt);
		return vElt;
	}
}
