/**
 * LICENCE[[
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1/CeCILL 2.O
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is kelis.fr code.
 *
 * The Initial Developer of the Original Code is 
 * samuel.monsarrat@kelis.fr
 *
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * sylvain.splinelli@kelis.fr
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * or the CeCILL Licence Version 2.0 (http://www.cecill.info/licences.en.html),
 * in which case the provisions of the GPL, the LGPL or the CeCILL are applicable
 * instead of those above. If you wish to allow use of your version of this file
 * only under the terms of either the GPL, the LGPL or the CeCILL, and not to allow
 * others to use your version of this file under the terms of the MPL, indicate
 * your decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL, the LGPL or the CeCILL. If you do not
 * delete the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL, the LGPL or the CeCILL.
 * ]]LICENCE
 */

/** SCENARI Slide-show slide manager 
 */
var scSlideMgr = {
	fBindBlks : [],
	fDefaultFontSize : 22, // fontSize in pixels
	fDefaultAnimStep : 5000,
	fBlocksPath : "des:.ssBkRoot",
	fStrings : {en:["previous","previous image",                   //0
	            "next","next image",                               //2
	            "play","play the animation",                       //4
	            "pause","pause the animation"],                    //6
               fr:["précédant","previous image",
	             "suivant","next image",
	             "lancer","lancer l'animation",
	             "arrêter","arrêter l'animation"],
               es:["anterior","imagen anterior",
	             "siguiente","imagen siguiente",
	             "arrancar","arrancar la animación",
	             "parar","parar la animación"]},
	fSsClassPrefix : "sld",
	
	/* === Public ============================================================= */
	init : function() {
	},
	setBlocksPath : function(pBlocksPath) {
		this.fBlocksPath = pBlocksPath;
	},
	/** Appelé par scPresMgr, pas par le framework standardscOnLoads. */
	onLoad : function() {
		try{
			//scCoLib.util.log("scSlideMgr.onLoad");
			this.fBlocksRoot = scPaLib.findNode(this.fBlocksPath, document.body);
			//Save init table col widths
			var vCols = scPaLib.findNodes("des:col");
			for (var i in vCols) if(!isNaN(vCols[i].width)) vCols[i].fDefaultWidth = vCols[i].width;
			//Set sizes
			this.xFixRatioNormalScreen();
			this.xFixBlocksRootSize();
			//Init des blocks
			for(var i=0; i < this.fBindBlks.length; i++) this.xBindBlk(this.fBindBlks[i].fPath, this.fBindBlks[i].fOpt, this.fBindBlks[i].fConst);
			//Abonnement au resize
			scSiLib.addRule(document.body, this);
			//Abonnement aux events souris
			if(scCoLib.isIE) document.body.attachEvent("onmousemove", this.sMouseMgr);
			else window.addEventListener("mousemove", this.sMouseMgr,true);
			if(scCoLib.isIE) document.body.attachEvent("onmousedown", this.sMouseMgr);
			else window.addEventListener("mousedown", this.sMouseMgr,true);
			//Abonnement aux events claviers
			document.onkeyup = this.sOnKeyUp;
		}catch(e){
			scCoLib.util.logError("ERROR scSlideMgr.onLoad : ", e);
		}
	},
	getBlocksRoot : function(){
		return this.fBlocksRoot;
	},
	getBlocksAvailHeight : function(){
		return this.fBlocksRoot.offsetHeight;
	},
	getBlocksAvailWidth : function(){
		return this.fBlocksRoot.offsetWidth;
	},
	getRatioNormalScreen : function(){
		return this.fRatioNormalScreen;
	},
	/** scSlideMgr.addBindableBlk : Add a bindable block for processing by onLoad(). */
	addBindableBlk : function(pPath, pOpt, pBkConstructor){
		this.fBindBlks.push({fPath : pPath, fOpt : pOpt, fConst : pBkConstructor});
	},
	/** scSlideMgr.findOwnerBlk : Retrun the block that is ancestor to the specified element. */
	findOwnerBlk : function(pElt){
		if (pElt == this.fBlocksRoot) return pElt;
		var vAncs = [pElt];
		vAncs = vAncs.concat(scPaLib.findNodes("anc:", pElt));
		for(var i=0; i < vAncs.length; i++) if (vAncs[i] == this.fBlocksRoot) return vAncs[i-1];
		return null;
	},
	/** Api scSiLib. */
	onResizedAnc : function(pOwnerNode, pEvent) {
		//Resize du slide.
		if(pEvent.phase==1) {
			this.xFixRatioNormalScreen();
			this.xFixBlocksRootSize();
		} else {
			window.parent.scPresMgr.redrawSlideZone();
		}
	},
	/** Api scSiLib. */
	onResizedDes : function(pOwnerNode, pEvent) {
	},
	/** Api scSiLib. */
	ruleSortKey : "AA",

	/* === Private ============================================================ */
	xBindBlk : function(pPath, pOpt, pBkConstructor){
		//scCoLib.util.log("scSlideMgr.xBindBlk: "+pPath+" - "+pOpt);
		var vBlks = scPaLib.findNodes(pPath, this.fBlocksRoot);
		for(var i=0; i < vBlks.length; i++) {
			new pBkConstructor(vBlks[i], pOpt);
		}
	},
	sMouseMgr : function(){
		if (window.frameElement && window.frameElement.fSldMgr) return window.frameElement.fSldMgr.mouseMgr();
	},
	sOnKeyUp : function(pEvent){
		var vEvent = pEvent || window.event;
		var vCharCode = vEvent.which || vEvent.keyCode;
		if (window.frameElement && window.frameElement.fSldMgr) return window.frameElement.fSldMgr.keyMgr(vCharCode);
	},
	xFixBlocksRootSize : function(){
		var vBodyH = this.xBodyHeight();
		var vTop = this.fBlocksRoot.offsetTop;
		this.fBlocksRoot.style.height = Math.max(50, vBodyH - vTop)+"px";
	},
	xBodyHeight : function() {
		if (window.innerHeight >= 0) {
			return window.innerHeight;
		} else if (document.documentElement) {
			return document.documentElement.clientHeight;
		} else if (document.body.clientHeight >= 0) {
			return document.body.clientHeight;
		} else {
			return 0;
		}
	},
	xBodyWidth : function() {
		if (window.innerHeight >= 0) {
			return window.innerWidth;
		} else if (document.documentElement) {
			return document.documentElement.clientWidth;
		} else if (document.body.clientWidth >= 0) {
			return document.body.clientWidth;
		} else {
			return 0;
		}
	},
	xFixRatioNormalScreen : function(){
		this.fRatioNormalScreen = Math.sqrt(this.xBodyHeight() / 600 * this.xBodyWidth() / 800);
		var vBaseFontSize = Math.round(this.fRatioNormalScreen * this.fDefaultFontSize);
		document.body.style.fontSize = vBaseFontSize+"px";
		// Adapt table col width
		var vCols = scPaLib.findNodes("des:col");
		for (var i in vCols) if(vCols[i].fDefaultWidth) vCols[i].width = Math.round(this.fRatioNormalScreen * vCols[i].fDefaultWidth);
	},

	/* === Utilities ========================================================== */
	/** scSlideMgr.xAddElt : Add an HTML element to a parent node. */
	xAddElt : function(pName, pParent, pClassName, pNoDisplay, pHidden, pNxtSib){
		return window.parent.scPresMgr.xAddElt(pName, pParent, pClassName, pNoDisplay, pHidden, pNxtSib);
	},
	/** scSlideMgr.xAddBtn : Add a HTML button to a parent node. */
	xAddBtn : function(pParent, pMgr, pClassName, pFunc, pCapt, pTitle, pNxtSib) {
		var vBtn = pParent.ownerDocument.createElement("a");
		vBtn.className = pClassName;
		vBtn.fName = pClassName;
		vBtn.href = "#";
		vBtn.target = "_self";
		vBtn.onclick = pFunc;
		vBtn.setAttribute("title", pTitle);
		vBtn.innerHTML = "<span>" + pCapt + "</span>"
		vBtn.fMgr = pMgr;
		if (pNxtSib) pParent.insertBefore(vBtn,pNxtSib)
		else pParent.appendChild(vBtn);
		return vBtn;
	},
	/** Set the opacity of a given node.
	 * @param pRate Variable de 0 à 1.
	 */
	xSetOpacity: function(pNode, pRate){
		window.parent.scPresMgr.xSetOpacity(pNode, pRate);
	},
	/** Start the opacity of a given node.
	 * On ajoute le filtre d'opacité sur IE.
	 * On place le node en visibility: "".
	 * @param pRate 2 valeurs possibles: 0 (invisible) ou 1 (visible).
	 */
	xStartOpacityEffect: function(pNode, pRate){
		window.parent.scPresMgr.xStartOpacityEffect(pNode, pRate);
	},
	/** End the opacity of a given node.
	 * On supprime le filtre d'opacité sur IE (évite des bugs de refresh).
	 * On place le node en visibility: hidden.
	 * @param pRate 2 valeurs possibles: 0 (invisible) ou 1 (visible).
	 */
	xEndOpacityEffect: function(pNode, pRate){
		window.parent.scPresMgr.xEndOpacityEffect(pNode, pRate);
	},
	/** Reteive a localized string. */
	xGetStr: function(pStrId) {
		var vLang = navigator.userLanguage || navigator.language;
		vLang = vLang.slice(0,2).toLowerCase();
		if (this.fStrings[vLang])	return this.fStrings[vLang][pStrId];
		else return this.fStrings["en"][pStrId];
	},
	/** scSlideMgr.xReadStyle : Read a style property (inline or CSS). */
	xReadStyle : function(pElt, pProp, pDoc) {
		var vDoc = pDoc || document;
		return window.parent.scPresMgr.xEndOpacityEffect(pElt, pProp, vDoc);
	}
}

/* ===== Animation Block class ================================================= */
/** scSlideMgr.SldAnimBlk : Animation block. */
scSlideMgr.SldAnimBlk = function(pElt, pOpt){
	this.fIdx = -1;
	this.fRateOld = [.9, .8, .7, .6, .5, .4, .3, .2, .1];
	this.fRateNew = [.1, .2, .3, .4, .5, .6, .7, .8, .9];
	this.fNxtSwitchTime = ( Date.now  ? Date.now() : new Date().getTime() ) + scSlideMgr.fDefaultAnimStep;
	this.fEndTime = this.fNxtSwitchTime + 100;
	this.fImgs = scPaLib.findNodes(this.fPathImgs, pElt);
	for (i in this.fImgs) {
		this.fImgs[i].style.position = "absolute";
		this.fImgs[i].style.visibility = "hidden";
		this.fImgs[i].fPrvImg = this.fImgs[(parseInt(i) > 0 ? parseInt(i)-1 : this.fImgs.length - 1)];
		this.fImgs[i].fNxtImg = this.fImgs[(parseInt(i) < this.fImgs.length - 1 ? parseInt(i)+1 : 0)];
	}
	this.fCurrImg = this.fImgs[0];
	this.fNxtImg = this.fCurrImg.fNxtImg;
	scSlideMgr.xStartOpacityEffect(this.fCurrImg, 1);
	scSlideMgr.xStartOpacityEffect(this.fNxtImg, 0);
	pElt.fImgs = this.fImgs;
	pElt.style.width="100%";

	pElt.fCtrl = scSlideMgr.xAddElt("div",pElt,scSlideMgr.fSsClassPrefix + "AnmCtrl");
	pElt.fBtnPrv = scSlideMgr.xAddBtn(pElt.fCtrl,this,scSlideMgr.fSsClassPrefix+"AnmBtnPrv",this.btnPrv,scSlideMgr.xGetStr(0),scSlideMgr.xGetStr(1));
	pElt.fBtnPly = scSlideMgr.xAddBtn(pElt.fCtrl,this,scSlideMgr.fSsClassPrefix+"AnmBtnPly",this.btnPly,scSlideMgr.xGetStr(4),scSlideMgr.xGetStr(5));
	pElt.fBtnPly.style.display = "none";
	pElt.fBtnPse = scSlideMgr.xAddBtn(pElt.fCtrl,this,scSlideMgr.fSsClassPrefix+"AnmBtnPse",this.btnPse,scSlideMgr.xGetStr(6),scSlideMgr.xGetStr(7));
	pElt.fBtnNxt = scSlideMgr.xAddBtn(pElt.fCtrl,this,scSlideMgr.fSsClassPrefix+"AnmBtnNxt",this.btnNxt,scSlideMgr.xGetStr(2),scSlideMgr.xGetStr(3));
	pElt.fCtrl.style.visibility = "hidden";
	pElt.fCtrl.fOn = false;
	pElt.onmouseover = this.showCtrl;
	pElt.fMgr = this;
	this.fElt = pElt;
	this.fPlyMode = 2;
	this.fNxtPlyMode = 2;
	scSiLib.addRule(pElt, this);
	scTiLib.addTaskAfterAll(this);
	this.onResizedAnc(pElt,{phase:1});
}
scSlideMgr.SldAnimBlk.prototype.fPathImgs = scPaLib.compilePath("chi:");
scSlideMgr.SldAnimBlk.prototype.onResizedAnc = function(pOwnerNode, pEvent){
	if(pEvent.phase==1) {
		try{
			var vMaxHeight = 0;
			for (i in pOwnerNode.fImgs){
				vMaxHeight = Math.max(vMaxHeight,pOwnerNode.fImgs[i].clientHeight);
			}
			for (i in pOwnerNode.fImgs){
				pOwnerNode.fImgs[i].style.marginTop = (vMaxHeight - pOwnerNode.fImgs[i].clientHeight) / 2 + "px"
				pOwnerNode.fImgs[i].style.marginLeft = (pOwnerNode.clientWidth - pOwnerNode.fImgs[i].clientWidth) / 2 + "px"
			}
			pOwnerNode.style.height=vMaxHeight + "px";
		}catch(e){
			scCoLib.util.logError("ERROR scSlideMgr.SldAnimBlk.onResizedAnc : ", e);
		}
	}
}
scSlideMgr.SldAnimBlk.prototype.btnPrv = function(){
	var vMgr = this.fMgr;
	vMgr.fNxtPlyMode = -1;
	var vAnim = vMgr.fElt;
	vAnim.fBtnPly.style.display="";
	vAnim.fBtnPse.style.display="none";
}
scSlideMgr.SldAnimBlk.prototype.btnPly = function(){
	var vMgr = this.fMgr;
	vMgr.fNxtPlyMode = 2;
	var vAnim = vMgr.fElt;
	vAnim.fBtnPly.style.display="none";
	vAnim.fBtnPse.style.display="";
}
scSlideMgr.SldAnimBlk.prototype.btnPse = function(){
	var vMgr = this.fMgr;
	vMgr.fNxtPlyMode = 0;
	var vAnim = vMgr.fElt;
	vAnim.fBtnPly.style.display="";
	vAnim.fBtnPse.style.display="none";
}
scSlideMgr.SldAnimBlk.prototype.btnNxt = function(){
	var vMgr = this.fMgr;
	vMgr.fNxtPlyMode = 1;
	var vAnim = vMgr.fElt;
	vAnim.fBtnPly.style.display="";
	vAnim.fBtnPse.style.display="none";
}
scSlideMgr.SldAnimBlk.prototype.showCtrl = function(){
	if (this.fOffProc) window.clearTimeout(this.fOffProc);
	if (!this.fCtrl.fOn){
		new window.parent.scPresMgr.FadeEltTask(this.fCtrl, 1);
		this.fCtrl.fOn = true;
	}
	var vAnm = this;
	this.fOffProc = window.setTimeout(function(){vAnm.fMgr.hideCtrl()}, 3000);
}
scSlideMgr.SldAnimBlk.prototype.hideCtrl = function(){
	if (this.fElt.fCtrl.fOn){
		new window.parent.scPresMgr.FadeEltTask(this.fElt.fCtrl, 0);
		this.fElt.fCtrl.fOn = false;
		this.fElt.fOffProc = null;
	}
}
scSlideMgr.SldAnimBlk.prototype.setNxtPlyMode = function(){
	var vNow = (Date.now ? Date.now() : new Date().getTime());
	var vAddTempo = (this.fPlyMode == 2 && this.fNxtPlyMode == 2);
	this.fPlyMode = this.fNxtPlyMode;
	this.fNxtPlyMode = (this.fPlyMode != 2 ? 0 : 2);
	if (this.fPlyMode != 0) {
		this.fNxtSwitchTime = vNow + (vAddTempo ? scSlideMgr.fDefaultAnimStep : 0);
		this.fEndTime = this.fNxtSwitchTime + 100;
		this.fNxtImg = (this.fPlyMode < 0 ? this.fCurrImg.fPrvImg : this.fCurrImg.fNxtImg);
	}
}
scSlideMgr.SldAnimBlk.prototype.execTask = function(){
	try{
		var vNow = (Date.now ? Date.now() : new Date().getTime());
		if(this.fPlyMode != 0 && this.fNxtSwitchTime < vNow){
			if (this.fIdx < 0) {
				scSlideMgr.xStartOpacityEffect(this.fCurrImg, 1);
				scSlideMgr.xStartOpacityEffect(this.fNxtImg, 0);
			}
			while(this.fEndTime < vNow && this.fIdx < this.fRateOld.length) {
				this.fIdx++;
				this.fEndTime += 100;
			}
			this.fIdx++;
			this.fEndTime += 100;
			if(this.fIdx >= this.fRateOld.length) {
				scSlideMgr.xEndOpacityEffect(this.fCurrImg, 0);
				scSlideMgr.xEndOpacityEffect(this.fNxtImg, 1);
				this.fCurrImg = this.fNxtImg;
				this.fIdx = -1;
				this.setNxtPlyMode();
				return true;
			}
			scSlideMgr.xSetOpacity(this.fCurrImg, this.fRateOld[this.fIdx]);
			scSlideMgr.xSetOpacity(this.fNxtImg, this.fRateNew[this.fIdx]);
		} else if (this.fPlyMode != this.fNxtPlyMode) this.setNxtPlyMode();
	}catch(e){
		scCoLib.util.logError("ERROR scSlideMgr.SldAnimBlk.execTask : ", e);
	}
	return true;
}

/* ===== SlideZoom Block class ================================================= */
/** scSlideMgr.SldZmBlk : SlideZoom block (anchors pointing to slides that need to be opened as a zoom). */
scSlideMgr.SldZmBlk = function(pElt, pOpt){
	pElt.onclick = this.onClickZoom;
}
scSlideMgr.SldZmBlk.prototype.onClickZoom = function(){
	if (!window.frameElement || !window.frameElement.fSldMgr) return false;
	var vSldMgr = window.frameElement.fSldMgr;
	if(vSldMgr) vSldMgr.showZoom(this);
	return false;
}

/* ===== Size Block classes ================================================= */

/** scSlideMgr.SizeBlk : Base size block. */
scSlideMgr.SizeBlk = function(){
}
scSlideMgr.SizeBlk.prototype.fPathResFra = scPaLib.compilePath("chi:div.resFra");
scSlideMgr.SizeBlk.prototype.fPathZoom = scPaLib.compilePath("des:.zoom");
scSlideMgr.SizeBlk.prototype.onResizedDes = function(pOwnerNode, pEvent){
}
scSlideMgr.SizeBlk.prototype.isEltNotAlone = function(pRoot, pEltAlone){
	if(pEltAlone == pRoot) return false;
	switch(pRoot.nodeType) {
		case 1 : 
			var vNm = pRoot.nodeName;
			if(vNm == "IMG" || vNm == "OBJECT" |vNm == "EMBED") return true;
			for(var vCh = pRoot.firstChild; vCh; vCh = vCh.nextSibling) if(this.isEltNotAlone(vCh, pEltAlone)) return true;
			break;
		case 3 :
			return (/\S/.test(pRoot.nodeValue));
	}
	return false;
}

/** scSlideMgr.SizeBlkImg : Image size block. */
scSlideMgr.SizeBlkImg = function(pElt, pOpt){
	//scCoLib.util.log("scSlideMgr.SizeBlkImg");
	if(!pElt) return;
	this.fOpt = pOpt || {};
	var vImg = scPaLib.findNode(this.fPathImg, pElt);
	if(!vImg) return;
	pElt.fImg = vImg;
	pElt.fResFra = scPaLib.findNode(this.fPathResFra, pElt);
	pElt.fOriW = vImg.width || 500;
	pElt.fOriH = vImg.height || 400;
	pElt.fListImg = scPaLib.findNodes(this.fPathListImg, pElt);
	pElt.fListImg.push({src:vImg.src, width:vImg.width, height:vImg.height});
	scSiLib.addRule(pElt, this);
	pElt.fZoom = scPaLib.findNode(this.fPathZoom, pElt);
	if(pElt.fZoom) {
		pElt.fZoom.onclick = this.onClickZoom;
		pElt.fZoom.fBlk = pElt;
	}
	this.onResizedAnc(pElt,{phase:1});
}
scSlideMgr.SizeBlkImg.prototype = new  scSlideMgr.SizeBlk();
scSlideMgr.SizeBlkImg.prototype.constructor = scSlideMgr.SizeBlkImg;
scSlideMgr.SizeBlkImg.prototype.fPathImg = scPaLib.compilePath("des:img.imgBase");
scSlideMgr.SizeBlkImg.prototype.fPathListImg = scPaLib.compilePath("chi:.imgOthers/chi:img");
scSlideMgr.SizeBlkImg.prototype.onResizedAnc = function(pOwnerNode, pEvent){
	if(pEvent.phase==1) {
		try{
			var vImg = pOwnerNode.fImg;
			if(!vImg) return;
			var vIsAlone = ! this.isEltNotAlone(scSlideMgr.findOwnerBlk(pOwnerNode), pOwnerNode);	
			var vRate = scSlideMgr.getRatioNormalScreen();
			var vWantedW = pOwnerNode.fOriW * vRate;
			var vWantedH = pOwnerNode.fOriH * vRate;
			var vMaxW = scSlideMgr.getBlocksAvailWidth() *  (this.fOpt.ratioWidth || .8);
			var vMaxH = scSlideMgr.getBlocksAvailHeight() * ( vIsAlone ? (this.fOpt.ratioHeightAlone || .9) : (this.fOpt.ratioHeight || .8) ) - (this.fOpt.captionHeight || 30);
			var vFinalW;
			var vZoomOn = false;
			if(vWantedW > vMaxW || vWantedH > vMaxH) {
				var vRH = vWantedH / vMaxH;
				vFinalW = (vWantedW / vMaxW > vRH) ? vMaxW : vWantedW / vRH;
				vZoomOn = true;
			} else {
				vFinalW = vWantedW;
			}
			var vList = pOwnerNode.fListImg;
			for (var i = 1, l = vList.length; i < l; i++) if(vList[i].width > vFinalW) break;
			var vImgSel = vList[i-1];
			vImg.src = vImgSel.src;
			vImg.width = vImgSel.width;
			vImg.height = vImgSel.height;
			if (pOwnerNode.fResFra){
				pOwnerNode.fResFra.style.width=vImg.width+"px";
				pOwnerNode.fResFra.style.height=vImg.height+"px";
			}
			if (pOwnerNode.fZoom) pOwnerNode.fZoom.style.display = (vZoomOn && i < l-1) ? "" : "none";
		}catch(e){
			scCoLib.util.logError("ERROR scSlideMgr.SizeBlkImg.onResizedAnc : ", e);
		}
	}
}
scSlideMgr.SizeBlkImg.prototype.onClickZoom = function(pEvent) {
	if (!window.frameElement || !window.frameElement.fSldMgr) return;
	var vBlk = this.fBlk;
	var vSldMgr = window.frameElement.fSldMgr;
	var vZmItm = vSldMgr.showZoom(vBlk.fImg);
	var vCt = vSldMgr.getZoomContainer();
	var vResizer = {
		onResizedDes : function(pOwnerNode, pEvent) {},
		onResizedAnc : function(pOwnerNode, pEvent) {
			if(pEvent && pEvent.phase==2) return;
			var vRate = scSlideMgr.getRatioNormalScreen() * 1.3; //On force un 30% de plus pour le zoom.
			var vWantedW = vBlk.fOriW * vRate;
			var vWantedH = vBlk.fOriH * vRate;
			var vMaxW = vCt.offsetWidth;
			var vMaxH = vCt.offsetHeight;
			var vFinalW;
			if(vWantedW > vMaxW || vWantedH > vMaxH) {
				var vRH = vWantedH / vMaxH;
				vFinalW = (vWantedW / vMaxW > vRH) ? vMaxW : vWantedW / vRH;
			} else {
				vFinalW = vWantedW;
			}
			var vList = vBlk.fListImg;
			for (var i = 1, l = vList.length; i < l; i++) if(vList[i].width > vFinalW) break;
			var vImgSel = vList[i-1];
			vZmItm.src = vImgSel.src;
			vZmItm.width = vImgSel.width;
			vZmItm.height = vImgSel.height;
			vZmItm.style.marginTop = Math.max(0, (vMaxH - vZmItm.height) / 2)+"px";
		}
	}
	scSiLib.addRule(vZmItm, vResizer);
	vResizer.onResizedAnc(vZmItm);
}

/** scSlideMgr.SizeBlkObj : Embeded object size block. */
scSlideMgr.SizeBlkObj = function(pElt, pOpt){
	if(!pElt) return;
	this.fOpt = pOpt || {};
	var vObj = scPaLib.findNode(this.fPathObject, pElt);
	if(!vObj) return;
	pElt.fObj = vObj;
	pElt.fResFra = scPaLib.findNode(this.fPathResFra, pElt);
	scSiLib.addRule(pElt, this);
	pElt.fZoom = scPaLib.findNode(this.fPathZoom, pElt);
	if(pElt.fZoom) {
		pElt.fZoom.onclick = this.onClickZoom;
		pElt.fZoom.fBlk = pElt;
	}
	this.onResizedAnc(pElt,{phase:1});
}
scSlideMgr.SizeBlkObj.prototype = new  scSlideMgr.SizeBlk();
scSlideMgr.SizeBlkObj.prototype.constructor = scSlideMgr.SizeBlkObj;
scSlideMgr.SizeBlkObj.prototype.fPathObject = scPaLib.compilePath("des:object");
scSlideMgr.SizeBlkObj.prototype.onResizedAnc = function(pOwnerNode, pEvent){
	if(pEvent.phase==1) {
		try{
			var vObj = pOwnerNode.fObj;
			if(!vObj) return;
			if(!vObj.oriW) vObj.oriW = vObj.width || 500;
			if(!vObj.oriH) vObj.oriH = vObj.height || 400;
			var vIsAlone = ! this.isEltNotAlone(scSlideMgr.findOwnerBlk(pOwnerNode), pOwnerNode);	
			var vRate = scSlideMgr.getRatioNormalScreen();
			var vWantedW = vObj.oriW * vRate;
			var vWantedH = vObj.oriH * vRate;
			var vMaxW = scSlideMgr.getBlocksAvailWidth() *  (this.fOpt.ratioWidth || .8);
			var vMaxH = scSlideMgr.getBlocksAvailHeight() * ( vIsAlone ? (this.fOpt.ratioHeightAlone || .9) : (this.fOpt.ratioHeight || .8) ) - (this.fOpt.captionHeight || 30);
			if(vWantedW > vMaxW || vWantedH > vMaxH) {
				var vRW = vWantedW / vMaxW;
				var vRH = vWantedH / vMaxH;
				if(vRW > vRH) {
					vObj.width = vMaxW;
					vObj.height = vWantedH / vRW;
				} else {
					vObj.width = vWantedW / vRH;
					vObj.height = vMaxH;
				}
				if(pOwnerNode.fZoom) pOwnerNode.fZoom.style.display = "";
			} else {
				vObj.width = vWantedW;
				vObj.height = vWantedH;
				if(pOwnerNode.fZoom) pOwnerNode.fZoom.style.display = "none";
			}
			if (pOwnerNode.fResFra){
				pOwnerNode.fResFra.style.width=vObj.width+"px";
				pOwnerNode.fResFra.style.height=vObj.height+"px";
			}
		}catch(e){
			scCoLib.util.logError("ERROR scSlideMgr.SizeBlkObj.onResizedAnc : ", e);
		}
	}
}
scSlideMgr.SizeBlkObj.prototype.onClickZoom = function(pEvent) {
	if (!window.frameElement || !window.frameElement.fSldMgr) return;
	var vBlk = this.fBlk;
	var vSldMgr = window.frameElement.fSldMgr;
	var vZmItm = vSldMgr.showZoom(vBlk.fObj);
	var vCt = vSldMgr.getZoomContainer();
	var vResizer = {
		onResizedDes : function(pOwnerNode, pEvent) {},
		onResizedAnc : function(pOwnerNode, pEvent) {
			if(pEvent && pEvent.phase==2) return;
			var vRate = scSlideMgr.getRatioNormalScreen() * 1.3; //On force un 30% de plus pour le zoom.
			var vWantedW = vBlk.fObj.oriW * vRate;
			var vWantedH = vBlk.fObj.oriH * vRate;
			var vMaxW = vCt.offsetWidth;
			var vMaxH = vCt.offsetHeight;
			if(vWantedW > vMaxW || vWantedH > vMaxH) {
				var vRW = vWantedW / vMaxW;
				var vRH = vWantedH / vMaxH;
				if(vRW > vRH) {
					vZmItm.width = vMaxW;
					vZmItm.height = vWantedH / vRW;
				} else {
					vZmItm.width = vWantedW / vRH;
					vZmItm.height = vMaxH;
				}
			} else {
				vZmItm.width = vWantedW;
				vZmItm.height = vWantedH;
			}
			vZmItm.style.marginTop = Math.max(0, (vMaxH - vZmItm.height) / 2)+"px";
		}
	}
	scSiLib.addRule(vZmItm, vResizer);
	vResizer.onResizedAnc(vZmItm);
}
